import os

from django.conf.urls.defaults import url, patterns

urlpatterns = patterns('object_permissions.views.groups',
    url(r'^group/(?P<id>\d+)/permissions/?$','user_permissions', name="group-permissions"),
    url(r'^group/(?P<id>\d+)/permissions/user/(?P<user_id>\d+)/?$','user_permissions', name="group-user-permissions"),
    url(r'^group/(?P<id>\d+)/permissions/all/?$','all_permissions', name="group-all-permissions"),
)

urlpatterns += patterns('object_permissions.views.permissions',
    # List all perms for a given user
    url(r'^user/(?P<id>\d+)/permissions/all/?$','all_permissions', name="user-all-permissions"),
    
    # add permissions on an object
    url(r'^user/(?P<user_id>\d+)/permissions/(?P<class_name>\w+)/?$','view_obj_permissions', name="user-add-permissions"),
    url(r'^group/(?P<group_id>\d+)/permissions/(?P<class_name>\w+)/?$','view_obj_permissions', name="group-add-permissions"),
    
    # edit permissions on an object
    url(r'^user/(?P<user_id>\d+)/permissions/(?P<class_name>\w+)/(?P<obj_id>\d+)/?$', \
        'view_obj_permissions', name="user-edit-permissions"),
    url(r'^group/(?P<group_id>\d+)/permissions/(?P<class_name>\w+)/(?P<obj_id>\d+)?$', \
        'view_obj_permissions', name="group-edit-permissions"),
)

urlpatterns += patterns('object_permissions.views.widgets',
    url(r'^user/search/?$', 'search_users', name='user-search')
)

#The following is used to serve up local media files like images
root = '%s/static' % os.path.dirname(os.path.realpath(__file__))
urlpatterns += patterns('',
    (r'^object_permissions_media/(?P<path>.*)', 'django.views.static.serve',\
     {'document_root':  root}),
)



from django.conf import settings
from django.db import models, IntegrityError
from django.contrib.auth.models import User

from object_permissions.registration import permission_map, user_has_perm

class ObjectPermBackend(object):
    supports_object_permissions = True
    supports_anonymous_user = True

    def __init__(self, *args, **kwargs):
        if hasattr(settings, 'ANONYMOUS_USER_ID'):
            id = settings.ANONYMOUS_USER_ID
            try:
                self.anonymous, new = User.objects.get_or_create(id=id,
                        username='anonymous')
            except IntegrityError:
                # Couldn't get the UID we were told to get, but we were still
                # told to get *an* anonymous user, so we'll make one. Note
                # that this could totally cause a second IntegrityError, which
                # we'll allow to propagate. That's fine; worse things have
                # happened, and it will hopefully LART the user sufficiently.
                self.anonymous, new = User.objects.get_or_create(
                        username='anonymous')
        else:
            self.anonymous = None

    def authenticate(self, username, password):
        """ Empty method, this backend does not authenticate users """
        return None

    def has_perm(self, user_obj, perm, obj=None):
        """
        Return whether the user has the given permission on the given object.
        """

        if not user_obj.is_authenticated():
            if self.anonymous:
                user_obj = self.anonymous
            else:
                return False

        if obj is None:
            return False

        return user_has_perm(user_obj, perm, obj, True)

    def get_all_permissions(self, user_obj, obj=None):
        """
        Get a list of all permissions for the user on the given object.

        This includes permissions given through groups.
        """

        if not user_obj.is_authenticated():
            if self.anonymous:
                user_obj = self.anonymous
            else:
                return []

        if obj is None or not isinstance(obj, models.Model):
            return []

        model = obj.__class__
        permissions = permission_map[model]

        q = permissions.objects.filter(
            models.Q(group__user=user_obj) | models.Q(user=user_obj),
            obj=obj)

        # Extract all set permissions from each row in the query.
        rv = set()
        for row in q:
            rv.update(field.name for field in row._meta.fields
                if isinstance(field, models.IntegerField)
                and getattr(row, field.name))

        return list(rv)

    def get_group_permissions(self, user_obj, obj=None):
        """
        Get a list of permissions for this user's groups on the given object.
        """

        if not user_obj.is_authenticated():
            if self.anonymous:
                user_obj = self.anonymous
            else:
                return []

        if obj is None or not isinstance(obj, models.Model):
            return []

        model = obj.__class__
        permissions = permission_map[model]

        q = permissions.objects.filter(groups__user=user_obj, obj=obj)

        # Extract all set permissions from each row in the query.
        rv = set()
        for row in q:
            rv.update(field.name for field in row._meta.fields
                if isinstance(field, models.BooleanField)
                and getattr(row, field.name))

        return list(rv)



from django.http import HttpResponse
from django.contrib.auth.models import User, Group
from django.utils import simplejson

def search_users(request):
    """ search users and groups and return results as json """
    if 'term' in request.GET:
        term = request.GET['term']
    else:
        term = None

    if 'pk' in request.GET:
        pk = request.GET['pk']
    else:
        pk = None

    limit = 10
    if request.GET.get("groups", 'True') == 'True':
        data = simplejson.dumps(search_users_and_groups(term, pk, limit))
    else:
        data = simplejson.dumps(search_users_only(term, pk, limit))
    return HttpResponse(data, mimetype="application/json")


def search_users_only(term=None, pk=None, limit=10):
    """
    Returns a list of the top N matches from Users with a name
    starting with term
 
    @param term: the term to search for
    @param limit: the number of results to return
    """
    if pk:
        users = User.objects.filter(id=int(pk)) 
    elif term:
        users = User.objects.filter(username__istartswith=term)
    else:
        users = User.objects.all()
     
    users = users.values('pk', 'username')

    if pk:
        query = users[0]['username']     
    elif term:
        query = term
    else:
        query = ""

    if limit: 
        users = users[:limit]
     
    # lable each item as a user
    f = 'user'
    users = [(i['username'], f, i['pk']) for i in users]
     
    # sort list and crop out all but the top [limit] results
    users = sorted(users, key=lambda x: x[0]) 
    users = users if len(users) < limit else users[:limit]
 
    return {
        'query':query,
        'results':users
    }

def search_users_and_groups(term=None, pk=None, limit=10):
    """
    Returns a list of the top N matches from Groups and Users with a name
    starting with term.
    Warning: Searching for users and groups using a primary key will return
    a match from both users AND groups

    @param term: the term to search for
    @param pk: the primary key of the user/group to search for
    @param limit: the number of results to return
    """
    if pk:
        users = User.objects.filter(id=int(pk))
        groups = Group.objects.filter(id=int(pk))
    elif term:
        users = User.objects.filter(username__istartswith=term)
        groups = Group.objects.filter(name__istartswith=term)
    else:
        users = User.objects.all()
        groups = Group.objects.all()

    users = users.values('pk', 'username')
    groups = groups.values('pk', 'name')

    if pk:
        query = ""
    elif term:
        query = term
    else:
        query = ""

    if limit:
        users = users[:limit]
        groups = groups [:limit]

    # label each item as either user or a group
    f = 'user'
    users = [(i['username'], f, i['pk']) for i in users]
    f = 'group'
    groups = [(i['name'], f, i['pk']) for i in groups]

    # merge lists together
    # then sort lists and crop out all but the top [limit] results
    merged = users + groups
    merged = sorted(merged, key=lambda x: x[0])
    merged = merged if len(merged) < limit else merged[:limit]

    return {
        'query':query,
        'results':merged
    }



def db_table_exists(table, cursor=None):
    """
    Determine if a table exists in the database

    cribbed from:
    https://gist.github.com/527113/307c2dec09ceeb647b8fa1d6d49591f3352cb034
    """

    try:
        if not cursor:
            from django.db import connection
            cursor = connection.cursor()
        if not cursor:
            raise Exception
        table_names = connection.introspection.get_table_list(cursor)
    except:
        raise Exception("unable to determine if the table '%s' exists" % table)
    else:
        return table in table_names


import django.dispatch

granted = django.dispatch.Signal(providing_args=["perm", "object"])
revoked = django.dispatch.Signal(providing_args=["perm", "object"])


# signals issues when a user has edited permissions or groups via a view
# provided by object permissions.  These signals differ from granted and revoked
# because they require the user (editor) that was editing the permissions.
#
# granted and revoked signals will still be sent if permissions are edited

# sent when a user has been added to a group
view_add_user = django.dispatch.Signal(providing_args=["editor", "user", "obj"])

# sent when a user has been remove from a group
view_remove_user = django.dispatch.Signal(providing_args=["editor", "user", "obj"])

# send when a user's permissions have been edited
view_edit_user = django.dispatch.Signal(providing_args=["editor", "user", "obj"])


from operator import or_
from warnings import warn

from django.conf import settings
from django.contrib.auth.models import User, Group
from django.core.exceptions import ObjectDoesNotExist
from django import db
from django.db import models, transaction
from django.db.models import Model, Q, Sum

from object_permissions.signals import granted, revoked


TESTING = settings.TESTING if hasattr(settings, 'TESTING') else False


"""
Registration functions.

This is the meat and gravy of the entire app.

In order to use permissions with a Model, that Model's permissions must be
registered in advance. Registration can only be done once per Model, at model
definition time, and must include all permissions for that Model.

>>> register(["spam", "eggs", "toast"], Breakfast)

Once registered, permissions may be set for any pairing of an instance of that
Model and an instance of a User or Group.

Technical tl;dr: Registration can only happen once because Object Permissions
dynamically creates new models to store the permissions for a specific model.
Since the dynamic models need to be database-backed, they can't be altered
once defined and they must be defined before validation. We'd like to offer
our sincerest assurance that, even though dynamic models are dangerous, our
highly trained staff has expertly wrestled these majestic, fascinating,
terrifying beasts into cages, and now they are yours to tame and own. Buy one
today!

...Okay, that got weird. But you get the point. Only register() a model once.
"""

class RegistrationException(Exception):
    pass


class UnknownPermissionException(Exception):
    pass


__all__ = (
    'register',
    'get_class',
    'grant', 'grant_group',
    'revoke', 'revoke_group',
    'get_user_perms', 'get_group_perms',
    'revoke_all', 'revoke_all_group',
    'set_user_perms', 'set_group_perms',
    'get_users', 'get_users_all', 'get_users_any',
    'get_groups', 'get_groups_all', 'get_groups_any',
    "user_has_any_perms", "group_has_any_perms",
    "user_has_all_perms", "group_has_all_perms",
    'get_model_perms',
    'filter_on_perms',
)

permission_map = {}
"""
A mapping of Models to Models. The key is a registered Model, and the value is
the Model that stores the permissions on that Model.
"""

permissions_for_model = {}
"""
A mapping of Models to lists of permissions defined for that model.
"""

class_names = {}
"""
A mapping of Class name to Class object
"""

params_for_model = {}
"""
A mapping of Models to their param dictionaries.
"""

forbidden = set([
    "full_clean",
    "clean_fields",
    "clean",
    "validate_unique",
    "save",
    "pk",
    "delete",
    "get_absolute_url",
])
"""
Names reserved by Django for Model instances.
"""

_DELAYED = []
def register(params, model, app_label=None):
    """
    Register permissions for a Model.

    The permissions should be a list of names of permissions, e.g. ["eat",
    "order", "pay"]. This function will insert a row into the permission table
    if one does not already exist.

    For backwards compatibility, this function can also take a single
    permission instead of a list. This feature should be considered
    deprecated; please fix your code if you depend on this.
    """

    if isinstance(params, (str, unicode)):
        warn("Using a single permission is deprecated!")
        perms = [params]
    
    if app_label is None:
        warn("Registration without app_label is deprecated!")
        warn("Adding %s permissions table to object_permission app.  Note that you may not be able to migrate with south" % model)
        app_label = "object_permissions"

    # REPACK - for backward compatibility repack list of perms as a dict
    if isinstance(params, (list, tuple)):
        perms = params
        params = {'perms':params}
    else:
        perms = params['perms']

    for perm in perms:
        if perm in forbidden:
            raise RegistrationException("Permission %s is a reserved name!")

    # REPACK - For backwards compatibility and flexibility with parameters,
    # repack permissions list to ensure it is a dict of dicts
    if isinstance(perms, (list, tuple)):
        # repack perm list as a dictionary
        repack = {}
        for perm in perms:
            repack[perm] = {}
        params['perms'] = repack

    try:
        return _register(params, model, app_label)
    except db.utils.DatabaseError:
        # there was an error, likely due to a missing table.  Delay this
        # registration.
        _DELAYED.append((params, model, app_label))


@transaction.commit_manually
def _register(params, model, app_label):
    """
    Real method for registering permissions.

    This method is private; please don't call it from outside code.
    This inner function is required because its logic must also be available
    to call back from _register_delayed for delayed registrations.
    """
    try:
        if model in permission_map:
            warn("Tried to double-register %s for permissions!" % model)
            return

        name = "%s_Perms" % model.__name__
        fields = {
            "__module__": "",
            # XXX user xor group null?
            "user": models.ForeignKey(User, null=True,
                related_name="%s_uperms" % model.__name__),
            "group": models.ForeignKey(Group, null=True,
                related_name="%s_gperms" % model.__name__),
            "obj": models.ForeignKey(model,
                related_name="operms"),
        }

        for perm in params['perms']:
            fields[perm] = models.IntegerField(default=0)

        fields["Meta"] = type('Meta', (object,), dict(app_label=app_label))

        perm_model = type(name, (models.Model,), fields)
        permission_map[model] = perm_model
        permissions_for_model[model] = params['perms']
        params_for_model[model] = params
        class_names[model.__name__] = model
        return perm_model
    except:
        transaction.rollback()
        raise
    finally:
        transaction.commit()


def _register_delayed(**kwargs):
    """
    Register all permissions that were delayed waiting for database tables to
    be created.

    Don't call this from outside code.
    """
    try:
        for args in _DELAYED:
            _register(*args)
        models.signals.post_syncdb.disconnect(_register_delayed)
    except db.utils.DatabaseError:
        # still waiting for models in other apps to be created
        pass


models.signals.post_syncdb.connect(_register_delayed)


if TESTING:
    # XXX Create test tables only when TEST mode.  These models will be used in
    # various unittests.  This is used so that we do add unneeded models in a
    # production deployment.
    from django.db import models
    class TestModel(models.Model):
        name = models.CharField(max_length=32)
    class TestModelChild(models.Model):
        parent = models.ForeignKey(TestModel, null=True)
    class TestModelChildChild(models.Model):
        parent = models.ForeignKey(TestModelChild, null=True)
        
    TEST_MODEL_PARAMS = {
        'perms' : {
            # perm with both params
            'Perm1': {
                'description':'The first permission',
                'label':'Perm One'
            },
            # perm with only description
            'Perm2': {
                'description':'The second permission',
            },
            # perm with only label
            'Perm3': {
                'label':'Perm Three'
            },
            # perm with no params
            'Perm4': {}
        },
        'url':'test_model-detail',
        'url-params':['name']
    }
    register(TEST_MODEL_PARAMS, TestModel, 'object_permissions')
    register(['Perm1', 'Perm2','Perm3','Perm4'], TestModelChild, 'object_permissions')
    register(['Perm1', 'Perm2','Perm3','Perm4'], TestModelChildChild, 'object_permissions')


def get_class(class_name):
    return class_names[class_name]


def grant(user, perm, obj):
    """
    Grant a permission to a User.
    """

    model = obj.__class__

    if perm not in get_model_perms(model):
        raise UnknownPermissionException(perm)

    permissions = permission_map[model]

    try:
        user_perms = permissions.objects.get(user=user, obj=obj)
    except permissions.DoesNotExist:
        user_perms = permissions(user=user, obj=obj)

    # XXX could raise FieldDoesNotExist
    if not getattr(user_perms, perm):
        setattr(user_perms, perm, True)
        user_perms.save()

        granted.send(sender=user, perm=perm, object=obj)


def grant_group(group, perm, obj):
    """
    Grant a permission to a Group.
    """

    model = obj.__class__
    if perm not in get_model_perms(model):
        raise UnknownPermissionException(perm)
    
    permissions = permission_map[model]

    try:
        group_perms = permissions.objects.get(group=group, obj=obj)
    except permissions.DoesNotExist:
        group_perms = permissions(group=group, obj=obj)

    # XXX could raise FieldDoesNotExist
    if not getattr(group_perms, perm):
        setattr(group_perms, perm, True)
        group_perms.save()

        granted.send(sender=group, perm=perm, object=obj)


def set_user_perms(user, perms, obj):
    """
    Set User permissions to exactly the specified permissions.
    """    
    if perms:
        model = obj.__class__
        permissions = permission_map[model]
        
        all_perms = dict((p, False) for p in get_model_perms(model))
        for perm in perms:
            all_perms[perm] = True
        
        try:
            user_perms = permissions.objects.get(user=user, obj=obj)
        except permissions.DoesNotExist:
            user_perms = permissions(user=user, obj=obj)
        
        for perm, enabled in all_perms.iteritems():
            if enabled and not getattr(user_perms, perm):
                granted.send(sender=user, perm=perm, object=obj)
            elif not enabled and getattr(user_perms, perm):
                revoked.send(sender=user, perm=perm, object=obj)
            setattr(user_perms, perm, enabled)
        
        user_perms.save()
    
    else:
        # removing all perms.
        revoke_all(user, obj)

    return perms


def set_group_perms(group, perms, obj):
    """
    Set group permissions to exactly the specified permissions.
    """
    if perms:
        model = obj.__class__
        permissions = permission_map[model]
        all_perms = dict((p, False) for p in get_model_perms(model))
        for perm in perms:
            all_perms[perm] = True
    
        try:
            group_perms = permissions.objects.get(group=group, obj=obj)
        except permissions.DoesNotExist:
            group_perms = permissions(group=group, obj=obj)
    
        for perm, enabled in all_perms.iteritems():
            if enabled and not getattr(group_perms, perm):
                granted.send(sender=group, perm=perm, object=obj)
            elif not enabled and getattr(group_perms, perm):
                revoked.send(sender=group, perm=perm, object=obj)
    
            setattr(group_perms, perm, enabled)
    
        group_perms.save()

    else:
        # removing all perms.
        revoke_all_group(group, obj)

    return perms


def revoke(user, perm, obj):
    """
    Revoke a permission from a User.
    """

    model = obj.__class__
    permissions = permission_map[model]

    try:
        user_perms = permissions.objects.get(user=user, obj=obj)

        if getattr(user_perms, perm):
            revoked.send(sender=user, perm=perm, object=obj)

            setattr(user_perms, perm, False)

            # If any permissions remain, save the model. Otherwise, remove it
            # from the table.
            if any(getattr(user_perms, p)
                    for p in get_model_perms(model)):
                user_perms.save()
            else:
                user_perms.delete()

    except ObjectDoesNotExist:
        # User didnt have permission to begin with; do nothing.
        pass


def revoke_group(group, perm, obj):
    """
    Revokes a permission from a Group.
    """

    model = obj.__class__
    permissions = permission_map[model]

    try:
        group_perms = permissions.objects.get(group=group, obj=obj)

        if getattr(group_perms, perm):
            revoked.send(sender=group, perm=perm, object=obj)

            setattr(group_perms, perm, False)

            # If any permissions remain, save the model. Otherwise, remove it
            # from the table.
            if any(getattr(group_perms, p)
                    for p in get_model_perms(model)):
                group_perms.save()
            else:
                group_perms.delete()

    except ObjectDoesNotExist:
        # Group didnt have permission to begin with; do nothing.
        pass


def revoke_all(user, obj):
    """
    Revoke all permissions from a User.
    """

    model = obj.__class__
    permissions = permission_map[model]

    try:
        user_perms = permissions.objects.get(user=user, obj=obj)

        for perm in get_model_perms(model):
            if getattr(user_perms, perm):
                revoked.send(sender=user, perm=perm, object=obj)

        user_perms.delete()
    except ObjectDoesNotExist:
        pass


def revoke_all_group(group, obj):
    """
    Revoke all permissions from a Group.
    """

    model = obj.__class__
    permissions = permission_map[model]

    try:
        group_perms = permissions.objects.get(group=group, obj=obj)

        for perm in get_model_perms(model):
            if getattr(group_perms, perm):
                revoked.send(sender=group, perm=perm, object=obj)

        group_perms.delete()
    except ObjectDoesNotExist:
        pass


def get_user_perms(user, obj, groups=True):
    """
    Return the permissions that the User has on the given object.
    """
    klass = obj.__class__
    permissions = permission_map[klass]
    try:
        fields = get_model_perms(klass)
        kwargs = {}
        for field in fields:
            kwargs[field] = Sum(field)
        if groups:
            q = permissions.objects.filter(Q(user=user) | Q(group__user=user))
        else:
            q = permissions.objects.filter(user=user)
        q = q.filter(obj=obj).aggregate(**kwargs)
        return [field for field, value in q.items() if value]

    except permissions.DoesNotExist:
        return []


def get_user_perms_any(user, klass, groups=True):
    """
    return permission types that the user has on a given Model
    """
    permissions = permission_map[klass]
    try:
        fields = get_model_perms(klass)
        kwargs = {}
        for field in fields:
            kwargs[field] = Sum(field)
        if groups:
            q = permissions.objects.filter(Q(user=user) | Q(group__user=user))
        else:
            q = permissions.objects.filter(user=user)
        q = q.aggregate(**kwargs)
        return [field for field, value in q.items() if value]

    except permissions.DoesNotExist:
        return []


def get_group_perms(group, obj, groups=True):
    """
    Return the permissions that the Group has on the given object.

    @param groups - does nothing, compatibility with user version
    """
    klass = obj.__class__
    permissions = permission_map[klass]
    try:
        fields = get_model_perms(klass)
        kwargs = {}
        for field in fields:
            kwargs[field] = Sum(field)
        q = permissions.objects.filter(group=group, obj=obj).aggregate(**kwargs)
        return [field for field, value in q.items() if value]

    except permissions.DoesNotExist:
        return []


def get_group_perms_any(group, klass):
    """
    return permission types that the user has on a given Model
    """
    permissions = permission_map[klass]
    try:
        fields = get_model_perms(klass)
        kwargs = {}
        for field in fields:
            kwargs[field] = Sum(field)
        q = permissions.objects.filter(group=group).aggregate(**kwargs)
        return [field for field, value in q.items() if value]

    except permissions.DoesNotExist:
        return []


def get_model_perms(model):
    """
    Return all available permissions for a model.

    This function accepts both Models and model instances.
    """

    if isinstance(model, models.Model):
        # Instance; get the class
        model = model.__class__
    elif not issubclass(model, models.Model):
        # Not a Model subclass
        raise RegistrationException(
            "%s is neither a model nor instance of one" % model)

    if model not in permissions_for_model:
        raise RegistrationException(
            "Tried to get permissions for unregistered model %s" % model)
    return permissions_for_model[model]


def user_has_perm(user, perm, obj, groups=True):
    """
    Check if a User has a permission on a given object.

    If groups is True, the permissions of all Groups containing the user
    will also be considered.

    Silently returns False in case of several errors:

     * The model is not registered for permissions
     * The permission does not exist on this model
    """
    model = obj.__class__
    try:
        perms = get_model_perms(model)
    except RegistrationException:
        return False

    if perm not in perms:
        # not a valid permission
        return False

    permissions = permission_map[model]

    d = {
        perm: True
    }

    if groups:
        return permissions.objects.filter(obj=obj, **d) \
            .filter(Q(user=user) | Q(group__user=user)) \
            .exists()
    else:
        return permissions.objects.filter(user=user, obj=obj, **d).exists()


def group_has_perm(group, perm, obj):
    """
    Check if a Group has a permission on a given object.

    Silently returns False in case of several errors:

     * The model is not registered for permissions
     * The permission does not exist on this model
    """

    model = obj.__class__
    try:
        permissions = permission_map[model]
    except KeyError:
        return False

    if perm not in get_model_perms(model):
        # not a valid permission
        return False

    d = {
            perm: True,
    }

    return permissions.objects.filter(group=group, obj=obj, **d).exists()


def user_has_any_perms(user, obj, perms=None, groups=True):
    """
    Check whether the User has *any* permission on the given object.
    """
    instance = isinstance(obj, (Model,))
    model = obj.__class__ if instance else obj
    try:
        permissions = permission_map[model]
    except KeyError:
        return False

    # create perm clause, or implicit any
    if perms:
        # create Q clauses out of perms and OR them all together
        q = reduce(or_, (Q(**{perm:True}) for perm in perms))
        base = permissions.objects.filter(q)
    else:
        base = permissions.objects

    if groups:
        base = base.filter(Q(user=user) | Q(group__user=user))
    else:
        base = base.filter(user=user)

    # select model or instance level query
    return base.filter(obj=obj).exists() if instance else base.exists()


def group_has_any_perms(group, obj, perms=None):
    """
    Check whether the Group has *any* permission on the given object.
    """
    instance = isinstance(obj, (Model,))
    model = obj.__class__ if instance else obj
    try:
        permissions = permission_map[model]
    except KeyError:
        return False

    # create perm clause, or implicit any
    if perms:
        # create Q clauses out of perms and OR them all together
        q = reduce(or_, (Q(**{perm:True}) for perm in perms))
        base = permissions.objects.filter(q, group=group)
    else:
        base = permissions.objects.filter(group=group)
    
    # select model or instance level query
    return base.filter(obj=obj).exists() if instance else base.exists()


def user_has_all_perms(user, obj, perms, groups=True):
    """
    Check whether the User has *all* permission on the given object.
    """
    instance = isinstance(obj, (Model,))
    model = obj.__class__ if instance else obj
    try:
        permissions = permission_map[model]
    except KeyError:
        return False

    # create base query requiring all permissions
    perm_clauses = {}
    for perm in perms:
        perm_clauses[perm] = True
    base = permissions.objects.filter(**perm_clauses)

    # select users or users+groups
    if groups:
        base = base.filter(Q(user=user) | Q(group__user=user))
    else:
        base = base.filter(user=user)
        
    # select model or instance level query
    return base.filter(obj=obj).exists() if instance else base.exists()


def group_has_all_perms(group, obj, perms):
    """
    Check whether the Group has *all* permission on the given object.
    
    @param group - group for which to check permissions
    @param obj - Model or Instance for which to check permissions on.
    @param perms - list of permissions that must be matched
    
    @return True if group has all permissions on an instance.  If a model class
    is given this returns True if the group has permissions on any instance of
    the model.
    """
    
    instance = isinstance(obj, (Model,))
    model = obj.__class__ if instance else obj
    try:
        permissions = permission_map[model]
    except KeyError:
        return False

    # create base query requiring all permissions
    perm_clauses = {}
    for perm in perms:
        perm_clauses[perm] = True
    base = permissions.objects.filter(group=group, **perm_clauses)

    # select model or instance level query
    return base.filter(obj=obj).exists() if instance else base.exists()
    

def get_users_any(obj, perms=None, groups=True):
    """
    Retrieve the list of Users that have any of the permissions on the given
    object.

    @param perms - perms to check, or None if match *any* perms
    @param groups - include users with permissions via groups
    """
    model = obj.__class__
    permissions = permission_map[model]

    perm_table = "%s_uperms__%%s" % model.__name__
    obj_table = "%s_uperms__obj" % model.__name__
    d = {
            obj_table: obj,
    }

    if perms:
        # create Q clauses out of perms and OR them all together
        q = reduce(or_, (Q(**{perm_table % perm:True}) for perm in perms))
        
        if groups:
            # handle groups by checking perms for any group users are in.
            #
            # Do this by creating separate user and group clauses that check
            # the right object with the right set of perms.  Combine the clauses
            # together like so:
            #     (obj AND perms) OR (group_obj AND group perms)
            
            group_perm_table = "groups__%s_gperms__%%s" % model.__name__
            group_obj_table = "groups__%s_gperms__obj" % model.__name__
            gperms = reduce(or_, (Q(**{group_perm_table % perm:True}) \
                                  for perm in perms))
            group_clause = Q(**{group_obj_table:obj}) & gperms
            return User.objects.filter((Q(**d) & q) | group_clause).distinct()
            
        return User.objects.filter(q, **d).distinct()
    
    if groups:
        # handle groups with *any* perm by adding a clause that checks for the
        # object via the groups table.  this give inherent group membership
        # check.
        group_obj_table = "groups__%s_gperms__obj" % model.__name__
        group_clause = Q(**{group_obj_table:obj})
        return User.objects.filter(Q(**d) | group_clause).distinct()
    
    return User.objects.filter(**d).distinct()


def get_users_all(obj, perms, groups=True):
    """
    Retrieve the list of Users that have all of the permissions on the given
    object.

    @param perms - perms to check
    @param groups - include users with permissions via groups
    """
    model = obj.__class__
    permissions = permission_map[model]

    perm_table = "%s_uperms__%%s" % model.__name__
    obj_table = "%s_uperms__obj" % model.__name__
    d = {
            obj_table: obj,
    }

    # add all perm clauses to the main clause dictionary.  This will cause them
    # all to be AND'd together.
    for perm in perms:
        d[perm_table % perm] = True
    
    if groups:
        # handle groups by checking perms for any group users are in.
        #
        # Do this by creating separate user and group clauses that check
        # the right object with the right set of perms.  Combine the clauses
        # together like so:
        #     (obj AND perms) OR (group_obj AND group perms)
        
        group_perm_table = "groups__%s_gperms__%%s" % model.__name__
        group_obj_table = "groups__%s_gperms__obj" % model.__name__
        group_clause = {group_obj_table: obj}
        for perm in perms:
            group_clause[group_perm_table % perm] = True
        return User.objects.filter(Q(**d) | Q(**group_clause)).distinct()

    return User.objects.filter(**d).distinct()


def get_users(obj, groups=True):
    """
    Retrieve the list of Users that have permissions on the given object.
    """
    
    return get_users_any(obj, groups=groups)


def get_groups_any(obj, perms=None):
    """
    Retrieve the list of Groups that have any of the permissions on the given
    object.

    @param perms - perms to check, or None to check for *any* perms
    """
    
    model = obj.__class__
    permissions = permission_map[model]

    perm_table = "%s_gperms__%%s" % model.__name__
    obj_table = "%s_gperms__obj" % model.__name__
    d = {
            obj_table: obj,
    }

    if perms:
        # create Q clauses out of perms and OR them all together
        q = reduce(or_, (Q(**{perm_table % perm:True}) for perm in perms))
        return Group.objects.filter(q, **d).distinct()
    
    return Group.objects.filter(**d).distinct()


def get_groups_all(obj, perms):
    """
    Retrieve the list of Groups that have all of the permissions on the given
    object.

    @param perms - perms to check
    """
    
    model = obj.__class__
    permissions = permission_map[model]

    perm_table = "%s_gperms__%%s" % model.__name__
    obj_table = "%s_gperms__obj" % model.__name__
    d = {
            obj_table: obj,
    }

    # add all perm clauses to the main clause dictionary.  This will cause them
    # all to be AND'd together.
    for perm in perms:
        d[perm_table % perm] = True
    
    return Group.objects.filter(**d).distinct()


def get_groups(obj):
    """
    Retrieve the list of Users that have permissions on the given object.
    """

    return get_groups_any(obj)


def perms_on_any(user, model, perms, groups=True):
    """
    Determine whether the user has any of the listed permissions on any instances of
    the Model.

    This function checks whether either user permissions or group permissions
    are set, inclusively, using logical OR.

    @param user: user who must have permissions
    @param model: model on which to filter
    @param perms: list of perms to match
    @return true if has perms on any instance of model
    
    @deprecated - replaced by user_has_any_perms()
    """
    warn('user.perms_on_any() deprecated in lieu of user.has_any_perms()', stacklevel=2)
    return user_has_any_perms(user, model, perms, groups)


def filter_on_perms(user, model, perms, groups=True):
    warn('user.filter_on_perms() deprecated in lieu of user.get_objects_any_perms()', stacklevel=2)
    return user_get_objects_any_perms(user, model, perms, groups)


def user_get_objects_any_perms(user, model, perms=None, groups=True, **related):
    """
    Make a filtered QuerySet of objects for which the User has any of the
    requested permissions, optionally including permissions inherited from
    Groups.

    @param user: user who must have permissions
    @param model: model on which to filter
    @param perms: list of perms to match
    @param groups: include perms the user has from membership in Groups
    @param related: kwargs for related models.  Each kwarg name should be a
    valid query argument, you may follow as many tables as you like and perms
    are optional  E.g. foo__bar=['xoo'], foo=None
    @return a queryset of matching objects
    """
    
    q = Q(operms__user=user)

    # optionally add groups
    if groups:
        q |= Q(operms__group__user=user)
    
    # optionally add specific perms
    if perms:
        # OR all user permission clauses together
        model_perms = get_model_perms(model)
        perm_clause = reduce(or_, (Q(**{"operms__%s" % perm: True}) \
                                   for perm in perms if perm in model_perms))
        q &= perm_clause

    # related fields are built as sub-clauses for each related field.  To follow
    # the relation we must add a clause that follows the relationship path to
    # the operms table for that model, and optionally include perms.
    if related:
        
        for field in related:
            # build user clause that follows relationship through operms to user
            clause = Q(**{'%s__operms__user'%field:user})
            perms = related[field]
            
            # optionally include groups
            if groups:
                clause |= Q(**{'%s__operms__group__user'%field:user})
            
            # optionally include specific perms.
            if perms:
                perm_field = '%s__operms__%%s' % field
                perm_clause = reduce(or_, (Q(**{perm_field % perm: True}) \
                                                for perm in perms))
                clause &= perm_clause
            
            #add finished query
            q |= clause

    # return objects query filtered by the intricate Q statement
    return model.objects.filter(q).distinct()


def group_get_objects_any_perms(group, model, perms=None, **related):
    """
    Make a filtered QuerySet of objects for which the Group has any of the 
    requested permissions.

    @param group: group who must have permissions
    @param model: model on which to filter
    @param perms: list of perms to match
    @param groups: include perms the user has from membership in Groups
    @return a queryset of matching objects
    """

    # base clause matches group
    q = Q(operms__group=group)

    # optionally add permissions
    if perms:
        # permissions specified, OR all user permission clauses together
        model_perms = get_model_perms(model)
        perm_clause = reduce(or_, (Q(**{"operms__%s" % perm: True}) \
                                   for perm in perms if perm in model_perms))
        q &= perm_clause
    
    # related fields are built as sub-clauses for each related field.  To follow
    # the relation we must add a clause that follows the relationship path to
    # the operms table for that model, and optionally include perms.
    if related:
        for field, perms in related.items():
            # build group clause that follows relationship
            clause = Q(**{'%s__operms__group'%field:group})
            
            # optionally include specific perms.
            if perms:
                perm_field = '%s__operms__%%s' % field
                perm_clause = reduce(or_, (Q(**{perm_field % perm: True}) \
                                                for perm in perms))
                clause &= perm_clause
            
            #add finished query
            q |= clause
    
    return model.objects.filter(q).distinct()


def user_get_objects_all_perms(user, model, perms, groups=True, **related):
    """
    Make a filtered QuerySet of objects for which the User has all requested
    permissions, optionally including permissions inherited from Groups.

    @param user: user who must have permissions
    @param model: model on which to filter
    @param perms: list of perms to match
    @param groups: include perms the user has from membership in Groups
    @return a queryset of matching objects
    """
    
    # create kwargs including all perms that must be matched
    perm_clause = {}
    for perm in perms:
        perm_clause['operms__%s' % perm] = True
    
    if groups:
        # must match either a user or group clause + one of the perm clauses
        user_clause = Q(operms__group__user=user) | Q(operms__user=user)
        q = user_clause & Q(**perm_clause)
    
    else:
        # must match user clause + all of the perm clauses
        q = Q(operms__user=user, **perm_clause)

    # related fields are built as sub-clauses for each related field.  To follow
    # the relation we must add a clause that follows the relationship path to
    # the operms table for that model, and optionally include perms.
    if related:
        
        for field in related:
            # build user clause that follows relationship through operms to user
            clause = Q(**{'%s__operms__user'%field:user})
            perms = related[field]
            
            # optionally include groups
            if groups:
                clause |= Q(**{'%s__operms__group__user'%field:user})
            
            # create kwargs including all perms that must be matched
            perm_clause = {}
            for perm in perms:
                perm_clause['operms__%s' % perm] = True
            clause &= Q(**perm_clause)
            
            #add finished query
            q &= clause

    # return objects query filtered by the intricate Q statement
    return model.objects.filter(q).distinct()


def group_get_objects_all_perms(group, model, perms, **related):
    """
    Make a filtered QuerySet of objects for which the User has all requested
    permissions, optionally including permissions inherited from Groups.

    @param group: group who must have permissions
    @param model: model on which to filter
    @param perms: list of perms to match
    @param groups: include perms the user has from membership in Groups
    @return a queryset of matching objects
    """

    # base clause matches group
    q = Q(operms__group=group)

    # create kwargs including all perms that must be matched
    perm_clause = {}
    for perm in perms:
        perm_clause['operms__%s' % perm] = True
    q &= Q(**perm_clause)
    
    # related fields are built as sub-clauses for each related field.  To follow
    # the relation we must add a clause that follows the relationship path to
    # the operms table for that model, and optionally include perms.
    if related:
        for field, perms in related.items():
            # build group clause that follows relationship
            q &= Q(**{'%s__operms__group'%field:group})
            
            # create kwargs including all perms that must be matched
            perm_clause = {}
            for perm in perms:
                perm_clause['operms__%s' % perm] = True
            q &= Q(**perm_clause)
    
    return model.objects.filter(q).distinct()


def user_get_all_objects_any_perms(user, groups=True):
    """
    Get all objects from all registered models that the user has any permission
    for.
    
    This method does not accept a list of permissions since in most cases
    permissions will not exist across all models.  If a permission didn't exist
    on any model then it would cause an error to be thrown.
    
    @param user - user to check perms for
    @param groups - include permissions through groups
    @return a dictionary mapping class to a queryset of objects
    """
    perms = {}
    for cls in permission_map:
        perms[cls] = user_get_objects_any_perms(user, cls, groups=groups)
    return perms


def group_get_all_objects_any_perms(group):
    """
    Get all objects from all registered models that the group has any permission
    for.
    
    This method does not accept a list of permissions since in most cases
    permissions will not exist across all models.  If a permission didn't exist
    on any model then it would cause an error to be thrown.
    
    @param group - group to check perms for
    @return a dictionary mapping class to a queryset of objects
    """
    perms = {}
    for cls in permission_map:
        perms[cls] = group_get_objects_any_perms(group, cls)
    return perms


def filter_on_group_perms(group, model, perms):
    """
    Make a filtered QuerySet of objects for which the Group has any
    permissions.

    @param usergroup: Group who must have permissions
    @param model: model on which to filter
    @param perms: list of perms to match
    @param clauses: additional clauses to be added to the queryset
    @return a queryset of matching objects
    """
    warn('group.filter_on_perms() deprecated in lieu of group.get_objects_any_perms()', stacklevel=2)
    return group_get_objects_any_perms(group, model, perms)


# make some methods available as bound methods
setattr(User, 'grant', grant)
setattr(User, 'revoke', revoke)
setattr(User, 'revoke_all', revoke_all)
setattr(User, 'has_object_perm', user_has_perm)
setattr(User, 'has_any_perms', user_has_any_perms)
setattr(User, 'has_all_perms', user_has_all_perms)
setattr(User, 'get_perms', get_user_perms)
setattr(User, 'get_perms_any', get_user_perms_any)
setattr(User, 'set_perms', set_user_perms)
setattr(User, 'get_objects_any_perms', user_get_objects_any_perms)
setattr(User, 'get_objects_all_perms', user_get_objects_all_perms)
setattr(User, 'get_all_objects_any_perms', user_get_all_objects_any_perms)

# deprecated
setattr(User, 'filter_on_perms', filter_on_perms)
setattr(User, 'perms_on_any', perms_on_any)

setattr(Group, 'grant', grant_group)
setattr(Group, 'revoke', revoke_group)
setattr(Group, 'revoke_all', revoke_all_group)
setattr(Group, 'has_perm', group_has_perm)
setattr(Group, 'has_any_perms', group_has_any_perms)
setattr(Group, 'has_all_perms', group_has_all_perms)
setattr(Group, 'get_perms', get_group_perms)
setattr(Group, 'get_perms_any', get_group_perms_any)
setattr(Group, 'set_perms', set_group_perms)
setattr(Group, 'get_objects_any_perms', group_get_objects_any_perms)
setattr(Group, 'get_objects_all_perms', group_get_objects_all_perms)
setattr(Group, 'get_all_objects_any_perms', group_get_all_objects_any_perms)

# deprecated
setattr(Group, 'filter_on_perms', filter_on_group_perms)



from django.contrib.auth.models import User
from django.template import Library

from object_permissions.models import Group
from object_permissions.registration import get_user_perms, get_users_all

register = Library()


@register.filter
def permissions(user, object):
    """
    Returns the list of permissions a user has on an object
    """
    if user:
        return user.get_perms(object, False)
    return []


@register.filter
def group_admin(user, group=None):
    """
    Returns True or False based on if the user is an admin for any Groups
    """
    if group:
        return user.is_superuser or user.has_perm(group, 'admin')

    return user.is_superuser or user.has_any_perms(Group, ['admin'])


@register.filter
def class_name(cls):
    """
    Returns name of class for a class object
    """
    return cls.__name__


@register.filter
def is_user(obj):
    """
    Returns True if obj is a user
    """
    return isinstance(obj, (User,))


@register.simple_tag
def number_group_admins(group):
    "Return number of users with admin perms for specified group"
    return get_users_all(group, ["admin",], False).count()


@register.simple_tag
def permalink(obj, display=None):
    """
    Return a link for an object if it as a get_absolute_url method.  Not all
    models will have this.  Models that do not have the method will be rendered
    as text
    """
    display = display if display else obj
    if hasattr(obj, 'get_absolute_url'):
        return '<a href="%s">%s</a>' % (obj.get_absolute_url(), display)
    else:
        return obj


from muddle.shots import register, TemplateMixer

# permissions
register('user-detail-tab', TemplateMixer('object_permissions/muddle/user_permissions.html'))
register('group-detail-tab', TemplateMixer('object_permissions/muddle/group_permissions.html'))

# group list
register('group-list-table-headers', TemplateMixer('object_permissions/muddle/group/group_headers.html'))
register('group-list-table-cells', TemplateMixer('object_permissions/muddle/group/group_row.html'))

# group users
register('group-users-table-headers', TemplateMixer('object_permissions/muddle/group/user_headers.html'))
register('group-users-table-cells', TemplateMixer('object_permissions/muddle/group/user_row.html'))


import json

from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User, Group
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext

from object_permissions import get_user_perms
from object_permissions.signals import view_edit_user
from object_permissions.views.permissions import ObjectPermissionForm


@login_required
def user_permissions(request, id, user_id=None):
    """
    Ajax call to update a user's permissions
    
    @param id: id of Group
    """
    editor = request.user
    group = get_object_or_404(Group, id=id)
    
    if not (editor.is_superuser or editor.has_perm('admin', group)):
        return HttpResponseForbidden('You do not have sufficient privileges')
    
    if request.method == 'POST':
        form = ObjectPermissionForm(Group, request.POST)
        if form.is_valid():
            form.update_perms()
            user = form.cleaned_data['user']
            
            # send signal
            view_edit_user.send(sender=editor, user=user, obj=group)
            
            # return html to replace existing user row
            url = reverse('group-permissions', args=[id])
            return render_to_response(
                "object_permissions/muddle/group/user_row.html",
                {'object':group, 'user_detail':user, 'url':url},
                context_instance=RequestContext(request))
        
        # error in form return ajax response
        content = json.dumps(form.errors)
        return HttpResponse(content, mimetype='application/json')
    
    # render a form for an existing user only
    form_user = get_object_or_404(User, id=user_id)
    data = {'permissions':get_user_perms(form_user, group),
            'obj':group, 'user':user_id}
    form = ObjectPermissionForm(Group, data)
    return render_to_response("object_permissions/permissions/form.html",
                {'form':form, 'obj':group, 'user_id':user_id,
                'url':reverse('group-permissions', args=[group.id])},
                context_instance=RequestContext(request))
    

@login_required
def all_permissions(request, id, \
                    template='object_permissions/permissions/objects.html', rest=False):
    """
    Generic view for displaying permissions on all objects.
    
    @param id: id of group
    @param template: template to render the results with, default is
    permissions/objects.html
    """
    user = request.user
    group = get_object_or_404(Group, pk=id)
    
    if not (user.is_superuser or group.user_set.filter(pk=user.pk).exists()):
        if not rest:
            return HttpResponseForbidden('You do not have sufficient privileges')
        else:
            return {'error':'You do not have sufficient privileges'}
    
    perm_dict = group.get_all_objects_any_perms()

    # exclude group permissions from this view, they are treated special
    try:
        del perm_dict[Group]
    except KeyError:
        pass

    # XXX repack perm_dict so that class names are used as keys instead of the
    # classes.  Django templates will automatically execute anything callable
    # if you try to use it, even classes!  #5619
    repacked = {}
    for cls, objs in perm_dict.items():
        repacked[cls.__name__] = objs

    if not rest:
        return render_to_response(template, \
            {'persona':group, 'perm_dict':repacked}, \
        context_instance=RequestContext(request),
    )
    else:
        return {'persona':group, 'perm_dict':repacked}



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def convert_table(self, table, perms):
        for row in table.objects.all():
            for perm in perms:
                row.__dict__[perm] = row.__dict__['%s_tmp'%perm]
            row.save()

    def revert_table(self, table, perms):
        for row in table.objects.all():
            for perm in perms:
                row.__dict__['%s_tmp'%perm] = row.__dict__[perm]
            row.save()

    def forwards(self, orm):
        self.convert_table(orm['object_permissions.group_perms'], ['admin'])
    
    def backwards(self, orm):
        self.revert_table(orm['object_permissions.group_perms'], ['admin'])
    
    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'object_permissions.group_perms': {
            'Meta': {'object_name': 'Group_Perms'},
            'admin': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'admin_tmp': ('django.db.models.fields.IntegerField', [], {'default': 'False'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'Group_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['auth.Group']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'Group_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'object_permissions.testmodel': {
            'Meta': {'object_name': 'TestModel'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '32'})
        },
        'object_permissions.testmodel_perms': {
            'Meta': {'object_name': 'TestModel_Perms'},
            'Perm1': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm2': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm3': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm4': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModel_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['object_permissions.TestModel']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModel_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'object_permissions.testmodelchild': {
            'Meta': {'object_name': 'TestModelChild'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['object_permissions.TestModel']", 'null': 'True'})
        },
        'object_permissions.testmodelchild_perms': {
            'Meta': {'object_name': 'TestModelChild_Perms'},
            'Perm1': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm2': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm3': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm4': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChild_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['object_permissions.TestModelChild']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChild_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'object_permissions.testmodelchildchild': {
            'Meta': {'object_name': 'TestModelChildChild'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['object_permissions.TestModelChild']", 'null': 'True'})
        },
        'object_permissions.testmodelchildchild_perms': {
            'Meta': {'object_name': 'TestModelChildChild_Perms'},
            'Perm1': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm2': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm3': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm4': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChildChild_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['object_permissions.TestModelChildChild']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChildChild_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        }
    }
    
    complete_apps = ['object_permissions']



from django.conf import settings
from django.contrib.auth.models import User, Group
from django.test import TestCase
from django.test.client import Client

from object_permissions import *
from object_permissions.registration import TestModel, TestModelChild, \
    TestModelChildChild, UnknownPermissionException
from object_permissions.signals import view_edit_user


__all__ = ('TestGroups','TestGroupViews')


# XXX set global vars to keep tests cleaner
perms = set(['Perm1', 'Perm2', 'Perm3', 'Perm4'])
user0 = None
user1 = None
object0 = None
object1 = None

class TestGroups(TestCase):

    def setUp(self):
        global user0, user1, object0, object1
        self.tearDown()
        
        User(id=1, username='anonymous').save()
        settings.ANONYMOUS_USER_ID=1
        
        user0 = User(id=2, username='tester0')
        user0.set_password('secret')
        user0.save()
        user1 = User(id=3, username='tester1')
        user1.set_password('secret')
        user1.save()
        
        object0 = TestModel.objects.create(name='test0')
        object0.save()
        object1 = TestModel.objects.create(name='test1')
        object1.save()
    
    def tearDown(self):
        global user0, user1, object0, object1

        User.objects.all().delete()
        TestModel.objects.all().delete()
        TestModelChild.objects.all().delete()
        TestModelChildChild.objects.all().delete()
        Group.objects.all().delete()

        user0 = None
        user1 = None
        object0 = None
        object1 = None

    def test_trivial(self):
        """ Test instantiating a Group """
        Group()

    def test_save(self, name='test', user=None):
        """ Test saving an Group """
        group = Group(name=name)
        group.save()
        
        if user:
            group.user_set.add(user)
        
        return group
    
    def test_permissions(self):
        """ Verify all model perms are created """
        self.assertTrue('admin' in get_model_perms(Group))
    
    def test_grant_group_permissions(self):
        """
        Test granting permissions to a Group
       
        Verifies:
            * granted properties are available via backend (has_perm)
            * granted properties are only granted to the specified user, object
              combinations
            * granting unknown permission raises error
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        # grant single property
        group0.grant('Perm1', object0)
        self.assertTrue(user0.has_perm('Perm1', object0))
        self.assertFalse(user0.has_perm('Perm1', object1))
        self.assertFalse(user1.has_perm('Perm1', object0))
        self.assertFalse(user1.has_perm('Perm1', object1))
        
        # grant property again
        group0.grant('Perm1', object0)
        self.assertTrue(user0.has_perm('Perm1', object0))
        self.assertFalse(user0.has_perm('Perm1', object1))
        self.assertFalse(user1.has_perm('Perm1', object0))
        self.assertFalse(user1.has_perm('Perm1', object1))
        
        # grant second property
        group0.grant('Perm2', object0)
        self.assertTrue(user0.has_perm('Perm1', object0))
        self.assertFalse(user0.has_perm('Perm1', object1))
        self.assertFalse(user1.has_perm('Perm1', object0))
        self.assertFalse(user1.has_perm('Perm1', object1))
        self.assertTrue(user0.has_perm('Perm2', object0))
        self.assertFalse(user0.has_perm('Perm2', object1))
        self.assertFalse(user1.has_perm('Perm2', object0))
        self.assertFalse(user1.has_perm('Perm2', object1))
        
        # grant property to another object
        group0.grant('Perm2', object1)
        self.assertTrue(user0.has_perm('Perm1', object0))
        self.assertFalse(user0.has_perm('Perm1', object1))
        self.assertFalse(user1.has_perm('Perm1', object0))
        self.assertFalse(user1.has_perm('Perm1', object1))
        self.assertTrue(user0.has_perm('Perm2', object0))
        self.assertTrue(user0.has_perm('Perm2', object1))
        self.assertFalse(user1.has_perm('Perm2', object0))
        self.assertFalse(user1.has_perm('Perm2', object1))
        
        # grant perms to other user
        group1.grant('Perm3', object0)
        self.assertTrue(user0.has_perm('Perm1', object0))
        self.assertFalse(user0.has_perm('Perm1', object1))
        self.assertFalse(user1.has_perm('Perm1', object0))
        self.assertFalse(user1.has_perm('Perm1', object1))
        self.assertTrue(user0.has_perm('Perm2', object0))
        self.assertTrue(user0.has_perm('Perm2', object1))
        self.assertFalse(user1.has_perm('Perm2', object0))
        self.assertFalse(user1.has_perm('Perm2', object1))
        self.assertTrue(user1.has_perm('Perm3', object0))
        
        def grant_unknown():
            group1.grant('UnknownPerm', object0)
        self.assertRaises(UnknownPermissionException, grant_unknown)
    
    def test_revoke_group_permissions(self):
        """
        Test revoking permissions from Groups
        
        Verifies:
            * revoked properties are removed
            * revoked properties are only removed from the correct Group/obj combinations
            * revoking property Group does not have does not give an error
            * revoking unknown permission raises error
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        # revoke perm when user has no perms
        revoke(group0, 'Perm1', object0)
        
        for perm in perms:
            group0.grant(perm, object0)
            group0.grant(perm, object1)
            group1.grant(perm, object0)
            group1.grant(perm, object1)
        
        # revoke single perm
        group0.revoke('Perm1', object0)
        self.assertEqual(set(['Perm2', 'Perm3', 'Perm4']), set(group0.get_perms(object0)))
        self.assertEqual(perms, set(group0.get_perms(object1)))
        self.assertEqual(perms, set(group1.get_perms(object0)))
        self.assertEqual(perms, set(group1.get_perms(object1)))
        
        # revoke a second perm
        group0.revoke('Perm3', object0)
        self.assertEqual(set(['Perm2', 'Perm4']), set(group0.get_perms(object0)))
        self.assertEqual(perms, set(group0.get_perms(object1)))
        self.assertEqual(perms, set(group1.get_perms(object0)))
        self.assertEqual(perms, set(group1.get_perms(object1)))
        
        # revoke from another object
        group0.revoke('Perm3', object1)
        self.assertEqual(set(['Perm2', 'Perm4']), set(group0.get_perms(object0)))
        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(group0.get_perms(object1)))
        self.assertEqual(perms, set(group1.get_perms(object0)))
        self.assertEqual(perms, set(group1.get_perms(object1)))
        
        # revoke from another user
        group1.revoke('Perm4', object0)
        self.assertEqual(set(['Perm2', 'Perm4']), set(group0.get_perms(object0)))
        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(group0.get_perms(object1)))
        self.assertEqual(set(['Perm1', 'Perm2', 'Perm3']), set(group1.get_perms(object0)))
        self.assertEqual(perms, set(group1.get_perms(object1)))
        
        # revoke perm user does not have
        group0.revoke('Perm1', object0)
        self.assertEqual(set(['Perm2', 'Perm4']), set(group0.get_perms(object0)))
        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(group0.get_perms(object1)))
        self.assertEqual(set(['Perm1', 'Perm2', 'Perm3']), set(group1.get_perms(object0)))
        self.assertEqual(perms, set(group1.get_perms(object1)))
        
        # revoke perm that does not exist
        group0.revoke('DoesNotExist', object0)
        self.assertEqual(set(['Perm2', 'Perm4']), set(group0.get_perms(object0)))
        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(group0.get_perms(object1)))
        self.assertEqual(set(['Perm1', 'Perm2', 'Perm3']), set(group1.get_perms(object0)))
        self.assertEqual(perms, set(group1.get_perms(object1)))
    
    def test_revoke_all_group(self):
        """
        Test revoking all permissions from a group
        
        Verifies
            * revoked properties are only removed from the correct user/obj combinations
            * revoking property user does not have does not give an error
            * revoking unknown permission raises error
        """
        group0 = self.test_save('TestGroup0')
        group1 = self.test_save('TestGroup1')
        
        for perm in perms:
            grant_group(group0, perm, object0)
            grant_group(group0, perm, object1)
            grant_group(group1, perm, object0)
            grant_group(group1, perm, object1)
        
        revoke_all_group(group0, object0)
        self.assertEqual([], get_group_perms(group0, object0))
        self.assertEqual(perms, set(get_group_perms(group0, object1)))
        self.assertEqual(perms, set(get_group_perms(group1, object0)))
        self.assertEqual(perms, set(get_group_perms(group1, object1)))
        
        revoke_all_group(group0, object1)
        self.assertEqual([], get_group_perms(group0, object0))
        self.assertEqual([], get_group_perms(group0, object1))
        self.assertEqual(perms, set(get_group_perms(group1, object0)))
        self.assertEqual(perms, set(get_group_perms(group1, object1)))
        
        revoke_all_group(group1, object0)
        self.assertEqual([], get_group_perms(group0, object0))
        self.assertEqual([], get_group_perms(group0, object1))
        self.assertEqual([], get_group_perms(group1, object0))
        self.assertEqual(perms, set(get_group_perms(group1, object1)))
        
        revoke_all_group(group1, object1)
        self.assertEqual([], get_group_perms(group0, object0))
        self.assertEqual([], get_group_perms(group0, object1))
        self.assertEqual([], get_group_perms(group1, object0))
        self.assertEqual([], get_group_perms(group1, object1))
    
    def test_set_perms(self):
        """
        Test setting perms to an exact set
        """
        group0 = self.test_save('TestGroup0')
        group1 = self.test_save('TestGroup1')
        perms1 = perms
        perms2 = set(['Perm1', 'Perm2'])
        perms3 = set(['Perm2', 'Perm3'])
        perms4 = []
        # grant single property
        set_group_perms(group0, perms1, object0)
        self.assertEqual(perms1, set(get_group_perms(group0, object0)))
        self.assertEqual([], get_group_perms(group0, object1))
        self.assertEqual([], get_group_perms(group1, object0))
        
        set_group_perms(group0, perms2, object0)
        self.assertEqual(perms2, set(get_group_perms(group0, object0)))
        self.assertEqual([], get_group_perms(group0, object1))
        self.assertEqual([], get_group_perms(group1, object0))
        
        set_group_perms(group0, perms3, object0)
        self.assertEqual(perms3, set(get_group_perms(group0, object0)))
        self.assertEqual([], get_group_perms(group0, object1))
        self.assertEqual([], get_group_perms(group1, object0))
        
        # remove perms
        set_group_perms(group0, perms4, object0)
        self.assertEqual(perms4, get_group_perms(group0, object0))
        self.assertFalse(group0.TestModel_gperms.filter(obj=object0).exists())
        self.assertEqual([], get_group_perms(group0, object1))
        self.assertEqual([], get_group_perms(group1, object0))
        
        set_group_perms(group0, perms2, object1)
        self.assertEqual(perms4, get_group_perms(group0, object0))
        self.assertEqual(perms2, set(get_group_perms(group0, object1)))
        self.assertEqual([], get_group_perms(group1, object0))
        
        set_group_perms(group1, perms1, object0)
        self.assertEqual(perms4, get_group_perms(group0, object0))
        self.assertEqual(perms2, set(get_group_perms(group0, object1)))
        self.assertEqual(perms1, set(get_group_perms(group1, object0)))

    def test_get_perms(self):
        """
        tests retrieving list of perms

        Verifies:
            * No Perms returns empty list
            * some perms returns just that list
            * all perms returns all perms
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user0)

        self.assertEqual([], group0.get_perms(object0))

        group0.grant('Perm1', object0)
        group1.grant('Perm3', object1)
        group1.grant('Perm4', object1)

        self.assertEqual(['Perm1'], group0.get_perms(object0))

        perms = group0.get_perms(object1)
        self.assertEqual(2, len(perms))
        self.assertEqual(set(['Perm3','Perm4']), set(perms))

    def test_group_get_perms_any(self):
        """
        tests retrieving list of perms across any instance of a model

        Verifies:
            * No Perms returns empty list
            * some perms returns just that list
            * all perms returns all perms
        """
        group0 = self.test_save('TestGroup0')
        group1 = self.test_save('TestGroup1')

        self.assertEqual([], group0.get_perms_any(TestModel))

        group0.grant('Perm1', object0)
        group0.grant('Perm3', object1)
        group0.grant('Perm4', object1)
        group1.grant('Perm2', object0)

        perms = group0.get_perms_any(TestModel)
        self.assertEqual(3, len(perms))
        self.assertEqual(set(['Perm1', 'Perm3', 'Perm4']), set(perms))

    def test_has_perm(self):
        """
        Additional tests for has_perms
        
        Verifies:
            * None object always returns false
            * Nonexistent perm returns false
            * Perm user does not possess returns false
        """
        group = self.test_save('TestGroup0', user0)
        group.grant('Perm1', object0)
        
        self.assertTrue(user0.has_perm('Perm1', object0))
        self.assertFalse(user0.has_perm('Perm1', None))
        self.assertFalse(user0.has_perm('DoesNotExist', object0))
        self.assertFalse(user0.has_perm('Perm2', object0))
    
    def test_group_has_perm(self):
        """
        Test Group.has_perm
        
        Verifies:
            * None object always returns false
            * Nonexistent perm returns false
            * Perm user does not possess returns false
        """
        group = self.test_save('TestGroup0', user0)
        group.grant('Perm1', object0)
        
        self.assertTrue(group.has_perm('Perm1', object0))
        self.assertFalse(group.has_perm('Perm1', None))
        self.assertFalse(group.has_perm('DoesNotExist', object0))
        self.assertFalse(group.has_perm('Perm2', object0))
    
    def test_group_has_any_perm(self):
        """
        Test group_has_any_perms.  Group having any of the listed perms
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user0)
        
        # no perms
        self.assertFalse(group_has_any_perms(group0, object0))
        self.assertFalse(group_has_any_perms(group0, object0, ['Perm1', 'Perm2']))
        
        # single perm
        group0.grant("Perm1", object0)
        group1.grant("Perm2", object0)
        self.assertTrue(group_has_any_perms(group0, object0))
        self.assertTrue(group_has_any_perms(group1, object0))
        self.assertTrue(group_has_any_perms(group0, object0, ['Perm1', 'Perm2']))
        self.assertTrue(group_has_any_perms(group1, object0, ['Perm1', 'Perm2']))
        group0.revoke_all(object0)
        group1.revoke_all(object0)
    
    def test_get_groups(self):
        """
        Tests retrieving list of Groups with perms on an object
        """
        group0 = self.test_save('TestGroup0')
        group1 = self.test_save('TestGroup1')
        
        group0.grant('Perm1', object0)
        group0.grant('Perm3', object1)
        group1.grant('Perm2', object1)
        
        self.assertTrue(group0 in get_groups(object0))
        self.assertFalse(group1 in get_groups(object0))
        self.assertTrue(group0 in get_groups(object1))
        self.assertTrue(group1 in get_groups(object1))
        self.assertTrue(len(get_groups(object1))==2)
    
    def test_get_groups_any(self):
        """
        Tests retrieving list of groups with perms on an object
        """
        group0 = self.test_save('TestGroup0')
        group1 = self.test_save('TestGroup1')
        
        group0.set_perms(['Perm1', 'Perm2'], object0)
        group0.set_perms(['Perm1', 'Perm3'], object1)
        group1.set_perms(['Perm2'], object1)
        
        # no perms
        self.assertFalse(user1 in get_groups_any(object0, ['Perm1']))
        
        # explicit any perms
        self.assertTrue(group0 in get_groups_any(object0))
        self.assertTrue(group0 in get_groups_any(object1))
        self.assertFalse(group1 in get_groups_any(object0))
        self.assertTrue(group1 in get_groups_any(object1))
        
        # has perms, but not the right one
        self.assertFalse(group0 in get_groups_any(object0, ['Perm3']))
        
        # has one perm, but not all
        self.assertTrue(group0 in get_groups_any(object0, ['Perm1','Perm3']))
        self.assertTrue(group0 in get_groups_any(object1, ['Perm1','Perm2']))
        
        # has single perm
        self.assertTrue(group0 in get_groups_any(object0, ['Perm1']))
        self.assertTrue(group0 in get_groups_any(object0, ['Perm2']))
        self.assertTrue(group1 in get_groups_any(object1, ['Perm2']))
        
        # has multiple perms
        self.assertTrue(group0 in get_groups_any(object0, ['Perm1','Perm2']))
        self.assertTrue(group0 in get_groups_any(object1, ['Perm1','Perm3']))
    
    def test_get_groups_all(self):
        """
        Tests retrieving list of groups with perms on an object
        """
        group0 = self.test_save('TestGroup0')
        group1 = self.test_save('TestGroup1')
        
        group0.set_perms(['Perm1', 'Perm2'], object0)
        group0.set_perms(['Perm1', 'Perm3'], object1)
        group1.set_perms(['Perm2'], object1)
        
        # no perms
        self.assertFalse(group1 in get_groups_all(object0, ['Perm1']))
        
        # has perms, but not the right one
        self.assertFalse(group0 in get_groups_all(object0, ['Perm3']))
        
        # has one perm, but not all
        self.assertFalse(group0 in get_groups_all(object0, ['Perm1','Perm3']))
        self.assertFalse(group0 in get_groups_all(object1, ['Perm1','Perm2']))
        
        # has single perm
        self.assertTrue(group0 in get_groups_all(object0, ['Perm1']))
        self.assertTrue(group0 in get_groups_all(object0, ['Perm2']))
        self.assertTrue(group1 in get_groups_all(object1, ['Perm2']))
        
        # has multiple perms
        self.assertTrue(group0 in get_groups_all(object0, ['Perm1','Perm2']))
        self.assertTrue(group0 in get_groups_all(object1, ['Perm1','Perm3']))

    def test_user_get_perms(self):
        """
        tests retrieving list of perms

        Verifies:
            * No Perms returns empty list
            * some perms returns just that list
            * all perms returns all perms
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user0)

        self.assertEqual([], user0.get_perms(object0))

        group0.grant('Perm1', object0)
        group1.grant('Perm3', object1)
        group1.grant('Perm4', object1)

        self.assertEqual(['Perm1'], user0.get_perms(object0))

        perms = user0.get_perms(object1)
        self.assertEqual(2, len(perms))
        self.assertEqual(set(['Perm3','Perm4']), set(perms))

        # test excluding groups
        self.assertEqual([], user0.get_perms(object0, False))


    def test_user_get_perms_any(self):
        """
        tests retrieving list of perms across any instance of a model

        Verifies:
            * No Perms returns empty list
            * some perms returns just that list
            * all perms returns all perms
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user0)

        self.assertEqual([], user0.get_perms_any(TestModel))

        group0.grant('Perm1', object0)
        group1.grant('Perm3', object1)
        group1.grant('Perm4', object1)

        perms = user0.get_perms_any(TestModel)
        self.assertEqual(3, len(perms))
        self.assertEqual(set(['Perm1', 'Perm3', 'Perm4']), set(perms))

        # exclude group perms
        self.assertEqual([], user0.get_perms_any(TestModel, False))

    def test_user_get_objects_any_perms(self):
        """
        Test filtering objects
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create(name='test2')
        object2.save()
        object3 = TestModel.objects.create(name='test3')
        object3.save()
        object4 = TestModel.objects.create(name='test4')
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm2', object1)
        group1.grant('Perm3', object2)
        group1.grant('Perm4', object3)
        user0.grant('Perm4', object4)
        
        # retrieve single perm
        self.assertTrue(object0 in user0.get_objects_any_perms(TestModel, ['Perm1']))
        self.assertTrue(object1 in user0.get_objects_any_perms(TestModel, ['Perm2']))
        self.assertTrue(object2 in user1.get_objects_any_perms(TestModel, ['Perm3']))
        self.assertTrue(object3 in user1.get_objects_any_perms(TestModel, ['Perm4']))
        
        # retrieve multiple perms
        query = user0.get_objects_any_perms(TestModel, ['Perm1', 'Perm2', 'Perm3'])
        self.assertTrue(object0 in query)
        self.assertTrue(object1 in query)
        self.assertEqual(2, query.count())
        query = user1.get_objects_any_perms(TestModel, ['Perm1', 'Perm3', 'Perm4'])
        self.assertTrue(object2 in query)
        self.assertTrue(object3 in query)
        self.assertEqual(2, query.count())
        
        # mix of group and users
        query = user0.get_objects_any_perms(TestModel, ['Perm1', 'Perm4'])
        self.assertTrue(object0 in query)
        self.assertTrue(object4 in query)
        self.assertEqual(2, query.count())
        
        # retrieve no results
        query = user0.get_objects_any_perms(TestModel, ['Perm3'])
        self.assertEqual(0, query.count())
        query = user1.get_objects_any_perms(TestModel, ['Perm1'])
        self.assertEqual(0, query.count())
        
        # extra kwargs
        query = user0.get_objects_any_perms(TestModel, ['Perm1', 'Perm2', 'Perm3']).filter(name='test0')
        self.assertTrue(object0 in query)
        self.assertEqual(1, query.count())
        
        # exclude groups
        query = user0.get_objects_any_perms(TestModel, ['Perm1', 'Perm4'], groups=False)
        self.assertTrue(object4 in query)
        self.assertEqual(1, query.count())
    
    def test_user_get_objects_any_perms_related(self):
        """
        Test retrieving objects with any matching perms and related model
        options
        """
        group0 = self.test_save('TestGroup0', user0)
        
        object2 = TestModel.objects.create(name='test2')
        object2.save()
        
        child0 = TestModelChild.objects.create(parent=object0)
        child1 = TestModelChild.objects.create(parent=object1)
        child2 = TestModelChild.objects.create(parent=object2)
        child3 = TestModelChild.objects.create(parent=object2)
        child0.save()
        child1.save()
        child2.save()
        
        childchild = TestModelChildChild.objects.create(parent=child0)
        childchild.save()
        
        group0.grant('Perm1', object0)  # perms on both
        group0.grant('Perm2', child0)   # perms on both
        group0.grant('Perm3', object1)  # perm on parent only (child 1)
        group0.grant('Perm4', child2)   # perm on child only
        group0.grant('Perm1', childchild)
        
        user0.grant('Perm1', object0)  # perms on both
        user0.grant('Perm2', child0)   # perms on both
        
        # related field with implicit perms
        query = user0.get_objects_any_perms(TestModelChild, parent=None)
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertTrue(child1 in query, 'user should have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms on parent, and directly')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        self.assertEqual(3, len(query))
        
        # related field with single perms
        query = user0.get_objects_any_perms(TestModelChild, parent=['Perm3'])
        
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertTrue(child1 in query, 'user should have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms on parent')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        self.assertEqual(3, len(query), query.values('id'))
        
        # related field with multiple perms
        query = user0.get_objects_any_perms(TestModelChild, parent=['Perm1','Perm3'])
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertTrue(child1 in query, 'user should have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms on parent')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        self.assertEqual(3, len(query))
        
        # mix of direct and related perms
        query = user0.get_objects_any_perms(TestModelChild, perms=['Perm4'], parent=['Perm1'])
        self.assertEqual(2, len(query))
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertFalse(child1 in query, 'user should not have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms directly')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        
        # multiple relations
        query = user0.get_objects_any_perms(TestModelChildChild, parent=['Perm2'], parent__parent=['Perm1'])
        self.assertTrue(childchild in query)
        self.assertEqual(1, len(query))
        
        # exclude groups
        query = user0.get_objects_any_perms(TestModelChild, groups=False, parent=['Perm1'])
        self.assertTrue(child0 in query)
        self.assertEqual(1, len(query))
    
    def test_user_has_any_perms_on_model(self):
        """
        Test checking if a user has any of the perms on any instance of the model
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create()
        object2.save()
        object3 = TestModel.objects.create()
        object3.save()
        object4 = TestModel.objects.create()
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm2', object1)
        group1.grant('Perm3', object2)
        user0.grant('Perm4', object4)
        
        # check single perm
        self.assertTrue(user0.has_any_perms(TestModel, ['Perm1']))
        self.assertTrue(user0.has_any_perms(TestModel, ['Perm2']))
        self.assertTrue(user1.has_any_perms(TestModel, ['Perm3']))
        
        # check multiple perms
        self.assertTrue(user0.has_any_perms(TestModel, ['Perm1', 'Perm4']))
        self.assertTrue(user0.has_any_perms(TestModel, ['Perm1', 'Perm2']))
        self.assertTrue(user1.has_any_perms(TestModel, ['Perm3', 'Perm4']))
        
        # no results
        self.assertFalse(user0.has_any_perms(TestModel, ['Perm3']))
        self.assertFalse(user1.has_any_perms(TestModel, ['Perm4']))
        
        # excluding group perms
        self.assertTrue(user0.has_any_perms(TestModel, ['Perm4'], False))
        self.assertTrue(user0.has_any_perms(TestModel, ['Perm2', 'Perm4'], False))
        self.assertFalse(user0.has_any_perms(TestModel, ['Perm2'], False))
    
    def test_user_has_all_perms_on_model(self):
        """
        Test checking if a user has all of the perms on any instance of the model
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create()
        object2.save()
        object3 = TestModel.objects.create()
        object3.save()
        object4 = TestModel.objects.create()
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm3', object0)
        group0.grant('Perm2', object1)
        group1.grant('Perm3', object2)
        user0.grant('Perm4', object4)
        
        # check single perm
        self.assertTrue(user0.has_all_perms(TestModel, ['Perm1']))
        self.assertTrue(user0.has_all_perms(TestModel, ['Perm2']))
        self.assertTrue(user1.has_all_perms(TestModel, ['Perm3']))
        
        # check multiple perms
        self.assertFalse(user0.has_all_perms(TestModel, ['Perm1', 'Perm4']))
        self.assertFalse(user0.has_all_perms(TestModel, ['Perm1', 'Perm2']))
        self.assertTrue(user0.has_all_perms(TestModel, ['Perm1', 'Perm3']))
        self.assertFalse(user1.has_all_perms(TestModel, ['Perm3', 'Perm4']))
        
        # no results
        self.assertFalse(user1.has_all_perms(TestModel, ['Perm4']))
        
        # excluding group perms
        self.assertTrue(user0.has_all_perms(TestModel, ['Perm4'], False))
        self.assertFalse(user0.has_all_perms(TestModel, ['Perm2', 'Perm4'], False))
        self.assertFalse(user0.has_all_perms(TestModel, ['Perm2'], False))
    
    def test_user_get_objects_all_perms_related(self):
        """
        Test retrieving objects with all matching perms and related model
        options
        """
        group0 = self.test_save('TestGroup0', user0)
        
        child0 = TestModelChild.objects.create(parent=object0)
        child1 = TestModelChild.objects.create(parent=object0)
        child2 = TestModelChild.objects.create(parent=object1)
        child0.save()
        child1.save()
        child2.save()
        
        childchild = TestModelChildChild.objects.create(parent=child0)
        childchild.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm1', object1)
        group0.grant('Perm2', object1)
        
        group0.grant('Perm1', child0)
        group0.grant('Perm1', child1)
        group0.grant('Perm2', child1)
        group0.grant('Perm1', childchild)
        
        user0.grant('Perm1', object0)  # perms on both
        user0.grant('Perm1', child0)   # perms on both
        
        # related field with single perms
        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm1'])
        self.assertEqual(2, len(query))
        self.assertTrue(child0 in query)
        self.assertTrue(child1 in query)
        self.assertFalse(child2 in query)
        
        # related field with single perms - has parent but not child
        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm4'], parent=['Perm1'])
        self.assertEqual(0, len(query))
        
        # related field with single perms - has child but not parent
        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm4'])
        self.assertEqual(0, len(query))
        
        # related field with multiple perms
        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm1','Perm2'])
        self.assertEqual(1, len(query))
        self.assertFalse(child0 in query)
        self.assertTrue(child1 in query)
        self.assertFalse(child2 in query)
        
        # multiple relations
        query = user0.get_objects_all_perms(TestModelChildChild, perms=['Perm1'], parent=['Perm1'], parent__parent=['Perm1'])
        self.assertEqual(1, len(query))
        self.assertTrue(childchild in query)
        
        # exclude groups
        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], groups=False, parent=['Perm1'])
        self.assertTrue(child0 in query)
        self.assertEqual(1, len(query))
    def test_get_perms(self):
        """
        tests retrieving list of perms across any instance of a model

        Verifies:
            * No Perms returns empty list
            * some perms returns just that list
            * all perms returns all perms
        """
        self.assertEqual([], user0.get_perms(object0))

        grant(user0, 'Perm1', object0)
        grant(user0, 'Perm3', object1)
        grant(user0, 'Perm4', object1)
        grant(user1, 'Perm2', object0)

        self.assertEqual(['Perm1'], user0.get_perms(object0))

        perms = user0.get_perms(object1)
        self.assertEqual(2, len(perms))
        self.assertEqual(set(['Perm3','Perm4']), set(perms))
    def test_group_has_any_perms_on_model(self):
        """
        Test checking if a user has any of the perms on any instance of the model
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create()
        object2.save()
        object3 = TestModel.objects.create()
        object3.save()
        object4 = TestModel.objects.create()
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm2', object1)
        group1.grant('Perm3', object2)
        group0.grant('Perm4', object4)
        
        # check single perm
        self.assertTrue(group0.has_any_perms(TestModel, ['Perm1']))
        self.assertTrue(group0.has_any_perms(TestModel, ['Perm2']))
        self.assertTrue(group1.has_any_perms(TestModel, ['Perm3']))
        
        # check multiple perms
        self.assertTrue(group0.has_any_perms(TestModel, ['Perm1', 'Perm4']))
        self.assertTrue(group0.has_any_perms(TestModel, ['Perm1', 'Perm2']))
        self.assertTrue(group1.has_any_perms(TestModel, ['Perm3', 'Perm4']))
        
        # no results
        self.assertFalse(group0.has_any_perms(TestModel, ['Perm3']))
        self.assertFalse(group1.has_any_perms(TestModel, ['Perm4']))
    
    def test_group_has_all_perms_on_model(self):
        """
        Test checking if a user has all of the perms on any instance of the model
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create()
        object2.save()
        object3 = TestModel.objects.create()
        object3.save()
        object4 = TestModel.objects.create()
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm3', object0)
        group0.grant('Perm2', object1)
        group1.grant('Perm3', object2)
        
        # check single perm
        self.assertTrue(group0.has_all_perms(TestModel, ['Perm1']))
        self.assertTrue(group0.has_all_perms(TestModel, ['Perm2']))
        self.assertTrue(group1.has_all_perms(TestModel, ['Perm3']))
        
        # check multiple perms
        self.assertFalse(group0.has_all_perms(TestModel, ['Perm1', 'Perm4']))
        self.assertFalse(group0.has_all_perms(TestModel, ['Perm1', 'Perm2']))
        self.assertTrue(group0.has_all_perms(TestModel, ['Perm1', 'Perm3']))
        self.assertFalse(group1.has_all_perms(TestModel, ['Perm3', 'Perm4']))
        
        # no results
        self.assertFalse(group1.has_all_perms(TestModel, ['Perm4']))
    
    def test_group_get_objects_any_perms(self):
        """
        Test filtering objects based only on the groups perms
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create(name='test2')
        object2.save()
        object3 = TestModel.objects.create(name='test3')
        object3.save()
        object4 = TestModel.objects.create(name='test4')
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm2', object1)
        group1.grant('Perm3', object2)
        group1.grant('Perm4', object3)
        
        # implicit any
        self.assertTrue(object0 in group0.get_objects_any_perms(TestModel, ['Perm1']))
        self.assertTrue(object1 in group0.get_objects_any_perms(TestModel, ['Perm2']))
        self.assertFalse(object2 in group0.get_objects_any_perms(TestModel, ['Perm2']))
        self.assertTrue(object2 in group1.get_objects_any_perms(TestModel, ['Perm3']))
        self.assertTrue(object3 in group1.get_objects_any_perms(TestModel, ['Perm4']))
        
        # retrieve single perm
        self.assertTrue(object0 in group0.get_objects_any_perms(TestModel, ['Perm1']))
        self.assertTrue(object1 in group0.get_objects_any_perms(TestModel, ['Perm2']))
        self.assertTrue(object2 in group1.get_objects_any_perms(TestModel, ['Perm3']))
        self.assertTrue(object3 in group1.get_objects_any_perms(TestModel, ['Perm4']))
        
        # retrieve multiple perms
        query = group0.get_objects_any_perms(TestModel, ['Perm1', 'Perm2', 'Perm3'])
        self.assertTrue(object0 in query)
        self.assertTrue(object1 in query)
        self.assertEqual(2, query.count())
        query = group1.get_objects_any_perms(TestModel, ['Perm1', 'Perm3', 'Perm4'])
        self.assertTrue(object2 in query)
        self.assertTrue(object3 in query)
        self.assertEqual(2, query.count())
        
        # retrieve no results
        query = group0.get_objects_any_perms(TestModel, ['Perm3'])
        self.assertEqual(0, query.count())
        query = group1.get_objects_any_perms(TestModel, ['Perm1'])
        self.assertEqual(0, query.count())
        
        # extra kwargs
        query = group0.get_objects_any_perms(TestModel, ['Perm1', 'Perm2', 'Perm3']).filter( name='test0')
        self.assertTrue(object0 in query)
        self.assertEqual(1, query.count())
    
    def test_group_get_objects_any_perms_related(self):
        """
        Test retrieving objects with any matching perms and related model
        options
        """
        group0 = self.test_save('TestGroup0', user0)
        
        object2 = TestModel.objects.create(name='test2')
        object2.save()
        
        child0 = TestModelChild.objects.create(parent=object0)
        child1 = TestModelChild.objects.create(parent=object1)
        child2 = TestModelChild.objects.create(parent=object2)
        child3 = TestModelChild.objects.create(parent=object2)
        child0.save()
        child1.save()
        child2.save()
        
        childchild = TestModelChildChild.objects.create(parent=child0)
        childchild.save()
        
        group0.grant('Perm1', object0)  # perms on both
        group0.grant('Perm2', child0)   # perms on both
        group0.grant('Perm3', object1)  # perm on parent only (child 1)
        group0.grant('Perm4', child2)   # perm on child only
        group0.grant('Perm1', childchild)
        
        # related field with implicit perms
        query = group0.get_objects_any_perms(TestModelChild, parent=None)
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertTrue(child1 in query, 'user should have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms on parent, and directly')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        self.assertEqual(3, len(query))
        
        # related field with single perms
        query = group0.get_objects_any_perms(TestModelChild, parent=['Perm3'])
        
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertTrue(child1 in query, 'user should have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms on parent')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        self.assertEqual(3, len(query), query.values('id'))
        
        # related field with multiple perms
        query = group0.get_objects_any_perms(TestModelChild, parent=['Perm1','Perm3'])
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertTrue(child1 in query, 'user should have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms on parent')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        self.assertEqual(3, len(query))
        
        # mix of direct and related perms
        query = group0.get_objects_any_perms(TestModelChild, perms=['Perm4'], parent=['Perm1'])
        self.assertEqual(2, len(query))
        self.assertTrue(child0 in query, 'user should have perms on parent and directly')
        self.assertFalse(child1 in query, 'user should not have perms on parent')
        self.assertTrue(child2 in query, 'user should have perms directly')
        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
        
        # multiple relations
        query = group0.get_objects_any_perms(TestModelChildChild, parent=['Perm2'], parent__parent=['Perm1'])
        self.assertTrue(childchild in query)
        self.assertEqual(1, len(query))
    
    def test_group_get_objects_all_perms(self):
        """
        Test filtering objects based only on the groups perms
        """
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create(name='test2')
        object2.save()
        object3 = TestModel.objects.create(name='test3')
        object3.save()
        object4 = TestModel.objects.create(name='test4')
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm2', object0)
        group0.grant('Perm4', object1)
        group1.grant('Perm3', object2)
        group1.grant('Perm4', object2)
        
        # retrieve single perm
        self.assertTrue(object0 in group0.get_objects_all_perms(TestModel, ['Perm1']))
        self.assertTrue(object1 in group0.get_objects_all_perms(TestModel, ['Perm4']))
        self.assertTrue(object2 in group1.get_objects_all_perms(TestModel, ['Perm3']))
        self.assertTrue(object2 in group1.get_objects_all_perms(TestModel, ['Perm4']))
        
        # retrieve multiple perms
        query = group0.get_objects_all_perms(TestModel, ['Perm1', 'Perm2'])
        self.assertTrue(object0 in query)
        self.assertFalse(object1 in query)
        self.assertEqual(1, query.count())
        query = group1.get_objects_all_perms(TestModel, ['Perm3', 'Perm4'])
        self.assertTrue(object2 in query)
        self.assertFalse(object3 in query)
        self.assertEqual(1, query.count())
        
        # retrieve no results
        self.assertFalse(group0.get_objects_all_perms(TestModel, ['Perm3']).exists())
        self.assertFalse(group0.get_objects_all_perms(TestModel, ['Perm1','Perm4']).exists())
        self.assertFalse(group1.get_objects_all_perms(TestModel, ['Perm1']).exists())
        
        # extra kwargs
        query = group0.get_objects_all_perms(TestModel, ['Perm1', 'Perm2']).filter( name='test0')
        self.assertTrue(object0 in query)
        self.assertEqual(1, query.count())
    
    def test_group_get_objects_all_perms_related(self):
        """
        Test retrieving objects with all matching perms and related model
        options
        """
        group0 = self.test_save('TestGroup0', user0)
        
        child0 = TestModelChild.objects.create(parent=object0)
        child1 = TestModelChild.objects.create(parent=object0)
        child2 = TestModelChild.objects.create(parent=object1)
        child0.save()
        child1.save()
        child2.save()
        
        childchild = TestModelChildChild.objects.create(parent=child0)
        childchild.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm1', object1)
        group0.grant('Perm2', object1)
        
        group0.grant('Perm1', child0)
        group0.grant('Perm1', child1)
        group0.grant('Perm2', child1)
        group0.grant('Perm1', childchild)
        
        # related field with single perms
        query = group0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm1'])
        self.assertEqual(2, len(query))
        self.assertTrue(child0 in query)
        self.assertTrue(child1 in query)
        self.assertFalse(child2 in query)
        
        # related field with single perms - has parent but not child
        query = group0.get_objects_all_perms(TestModelChild, perms=['Perm4'], parent=['Perm1'])
        self.assertEqual(0, len(query))
        
        # related field with single perms - has child but not parent
        query = group0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm4'])
        self.assertEqual(0, len(query))
        
        # related field with multiple perms
        query = group0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm1','Perm2'])
        self.assertEqual(1, len(query))
        self.assertFalse(child0 in query)
        self.assertTrue(child1 in query)
        self.assertFalse(child2 in query)
        
        # multiple relations
        query = group0.get_objects_all_perms(TestModelChildChild, perms=['Perm1'], parent=['Perm1'], parent__parent=['Perm1'])
        self.assertEqual(1, len(query))
        self.assertTrue(childchild in query)
    
    def test_group_get_all_objects_any_perms(self):
        group0 = self.test_save('TestGroup0', user0)
        group1 = self.test_save('TestGroup1', user1)
        
        object2 = TestModel.objects.create(name='test2')
        object2.save()
        object3 = TestModel.objects.create(name='test3')
        object3.save()
        object4 = TestModel.objects.create(name='test4')
        object4.save()
        
        group0.grant('Perm1', object0)
        group0.grant('Perm2', object1)
        group0.grant('Perm4', object1)
        
        perm_dict = group0.get_all_objects_any_perms()
        self.assertTrue(isinstance(perm_dict, (dict,)))
        self.assertTrue(TestModel in perm_dict, perm_dict.keys())
        self.assertTrue(object0 in perm_dict[TestModel])
        self.assertTrue(object1 in perm_dict[TestModel])
        self.assertFalse(object2 in perm_dict[TestModel])
        self.assertFalse(object3 in perm_dict[TestModel])
        self.assertFalse(object4 in perm_dict[TestModel])
        
        # no perms
        perm_dict = group1.get_all_objects_any_perms()
        self.assertTrue(isinstance(perm_dict, (dict,)))
        self.assertTrue(TestModel in perm_dict, perm_dict.keys())
        self.assertEqual(0, perm_dict[TestModel].count())


class TestGroupViews(TestCase):

    def setUp(self):
        global user0, user1, object0, object1
        self.tearDown()
        
        User(id=1, username='anonymous').save()
        settings.ANONYMOUS_USER_ID=1
        
        user0 = User(id=2, username='tester0')
        user0.set_password('secret')
        user0.save()
        user1 = User(id=3, username='tester1')
        user1.set_password('secret')
        user1.save()

        object0 = TestModel.objects.create(id=10, name='test0')
        object0.save()
        object1 = TestModel.objects.create(id=13, name='test1')
        object1.save()
    
    def tearDown(self):
        global user0, user1, object0, object1
        User.objects.all().delete()
        TestModel.objects.all().delete()
        Group.objects.all().delete()

        user0 = None
        user1 = None
        object0 = None
        object1 = None

    def test_save(self, name='test'):
        """ Test saving an Group """
        group = Group(name=name)
        group.save()
        return group
    
    def test_view_update_permissions(self):
        """
        Tests setting permissions for a user
        
        Verifies:
            * request from unauthorized user results in 403
            * GET returns a 200 code, response is html
            * POST with a user id adds user, response is html for user
            * POST without user_id or group_id returns error as json
            * POST with both a user_id and group_id returns error as json
            * POST for invalid user id returns error as json
            * POST for invalid group_id returns error as json
            * adding user a second time returns error as json
            * POST with a user_id adds user, response is html for user
            * POST with a group_id adds user, response is html for user
            * perms added to appropriate models
        """
        group = self.test_save()
        group.user_set.add(user0)
        group1 = self.test_save('other_group')
        
        c = Client()
        url = '/group/%d/permissions/user/%s/'
        url_post = '/group/%d/permissions/'
        args = (group.id, user0.id)
        args_post = group.id
        
        # anonymous user
        response = c.get(url % args, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, 'registration/login.html')
        
        # unauthorized
        self.assertTrue(c.login(username=user0.username, password='secret'))
        response = c.get(url % args)
        self.assertEqual(403, response.status_code)
        response = c.post(url % args)
        self.assertEqual(403, response.status_code)
        
        # authorized post (perm granted)
        grant(user0, 'admin', group)
        response = c.get(url % args, {'user':user0.id, 'obj':group.pk})
        self.assertEqual(200, response.status_code)
        self.assertEquals('text/html; charset=utf-8', response['content-type'])
        self.assertTemplateUsed(response, 'object_permissions/permissions/form.html')
        
        # authorized post (superuser)
        revoke(user0, 'admin', group)
        user0.is_superuser = True
        user0.save()
        response = c.get(url % args, {'user':user0.id})
        self.assertEqual(200, response.status_code)
        self.assertEquals('text/html; charset=utf-8', response['content-type'])
        self.assertTemplateUsed(response, 'object_permissions/permissions/form.html')
    
        # invalid user (GET)
        response = c.get(url % (group.id, -1))
        self.assertEqual(404, response.status_code)
        
        # invalid group (GET)
        response = c.get(url % (-1, user0.id))
        self.assertEqual(404, response.status_code)
        
        # invalid user (POST)
        data = {'permissions':['admin'], 'user':-1, 'obj':group.pk}
        response = c.post(url_post % args_post, data)
        self.assertEqual(200, response.status_code)
        self.assertEquals('application/json', response['content-type'])
        self.assertNotEquals('1', response.content)
        
        # invalid group (POST)
        data = {'permissions':['admin'], 'group':-1, 'obj':group.pk}
        response = c.post(url_post % args_post, data)
        self.assertEqual(200, response.status_code)
        self.assertEquals('application/json', response['content-type'])
        self.assertNotEquals('1', response.content)
        
        # user and group (POST)
        data = {'permissions':['admin'], 'user':user0.id, 'group':group1.id, 'obj':group.pk}
        response = c.post(url_post % args_post, data)
        self.assertEqual(200, response.status_code)
        self.assertEquals('application/json', response['content-type'])
        self.assertNotEquals('1', response.content)
        
        # invalid permission
        data = {'permissions':['DoesNotExist'], 'user':user0.id, 'obj':group.pk}
        response = c.post(url_post % args_post, data)
        self.assertEqual(200, response.status_code)
        self.assertEquals('application/json', response['content-type'])
        self.assertNotEquals('1', response.content)
        
        # setup signal
        self.signal_sender = self.signal_user = self.signal_obj = None
        def callback(sender, user, obj, **kwargs):
            self.signal_sender = sender
            self.signal_user = user
            self.signal_obj = obj
        view_edit_user.connect(callback)
        
        # valid post user

        data = {'permissions':['admin'], 'user':user0.id, 'obj':group.pk}
        response = c.post(url_post % args_post, data)

        self.assertEqual(200, response.status_code)
        self.assertEquals('text/html; charset=utf-8', response['content-type'])
        self.assertTrue(user0.has_perm('admin', group))
        self.assertEqual(['admin'], get_user_perms(user0, group))
        
        # check signal fired with correct values
        self.assertEqual(self.signal_sender, user0)
        self.assertEqual(self.signal_user, user0)
        self.assertEqual(self.signal_obj, group)
        view_edit_user.disconnect(callback)
        
        # valid post no permissions user
        data = {'user':user0.id, 'obj':group.pk}
        response = c.post(url_post % args_post, data)
        self.assertEquals('text/html; charset=utf-8', response['content-type'])
        self.assertEqual(200, response.status_code)
        self.assertEqual([], get_user_perms(user0, group))

    def test_permissions_all(self):
        """ tests groups.permissions_all() """
        group = self.test_save()
        group.user_set.add(user0)
        group1 = self.test_save('other_group')
        
        url = '/group/%s/permissions/all'
        c = Client()
        
        # anonymous user
        response = c.get(url % group.pk, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, 'registration/login.html')
        
        # unauthorized user - wrong group
        self.assertTrue(c.login(username=user0.username, password='secret'))
        response = c.get(url % group1.pk)
        self.assertEqual(403, response.status_code)
        
        # unknown group
        response = c.get(url % 123456)
        self.assertEqual(404, response.status_code)
        
        # authorized user - group member
        self.assertTrue(c.login(username=user0.username, password='secret'))
        response = c.get(url % group.pk)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, 'object_permissions/permissions/objects.html')
        
        # superuser
        user0.is_superuser = True
        user0.save()
        response = c.get(url % group1.pk)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, 'object_permissions/permissions/objects.html')



import json

from django import forms
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseNotFound, HttpResponseForbidden
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext

from object_permissions import get_user_perms, get_group_perms, \
    get_model_perms, get_users, get_groups, get_class
from object_permissions.models import Group
from object_permissions.signals import view_add_user, view_remove_user, \
    view_edit_user


class ObjectPermissionForm(forms.Form):
    """
    Form used for editing permissions
    """
    permissions = forms.MultipleChoiceField(required=False,
                                            widget=forms.CheckboxSelectMultiple)
    user = forms.ModelChoiceField(queryset=User.objects.all(), required=False)
    group = forms.ModelChoiceField(queryset=Group.objects.all(),
                                   required=False)
    obj = forms.ModelChoiceField(queryset=None)
    
    # dictionary used for caching the choices for specific models
    choices = {}
    
    def __init__(self, model, *args, **kwargs):
        """
        @param model - the object being granted permissions
        """
        super(ObjectPermissionForm, self).__init__(*args, **kwargs)
        
        self.model = model
        self.fields['obj'].queryset = model.objects.all()
        self.fields['obj'].label = model.__name__
        self.fields['permissions'].choices = self.get_choices(model)

    @classmethod
    def get_choices(cls, model):
        """
        helper method for getting choices for a model.  This method uses an
        internal cache to store the choices.
        
        @param model - Model class to fetch choices for
        """
        try: 
            return ObjectPermissionForm.choices[model]
        except KeyError:
            # choices weren't built yet.
            choices = []
            model_perms = get_model_perms(model)
            
            for perm, params in model_perms.items():
                display = params.copy()
                if 'label' not in display:
                    display['label'] = perm
                choices.append((perm, display))
            ObjectPermissionForm.choices[model] = choices
            return choices

    def clean(self):
        """
        validates:
            * mutual exclusivity of user and group
            * a user or group is always selected and set to 'grantee'
        """
        data = self.cleaned_data
        user = data.get('user')
        group = data.get('group')
        if not (user or group) or (user and group):
            raise forms.ValidationError('Choose a group or user')
        
        # add whichever object was selected
        data['grantee'] = user if user else group
        return data
    
    def update_perms(self):
        """
        updates perms for the user based on values passed in
            * grant all perms selected in the form.  Revoke all
            * other available perms that were not selected.
            
        @return list of perms the user now possesses
        """
        perms = self.cleaned_data['permissions']
        grantee = self.cleaned_data['grantee']
        obj = self.cleaned_data['obj']
        grantee.set_perms(perms, obj)
        return perms
    

class ObjectPermissionFormNewUsers(ObjectPermissionForm):
    """
    A subclass of permission form that enforces an addtional rule that new users
    must be granted at least one permission.  This is used for objects that
    determine group membership (e.g. listing users with acccess) based on who
    has permissions.
    
    This is different from objects that grant inherent permissions through a
    different membership relationship (e.g. Users in a Group inherit perms)
    """
    
    def clean(self):
        data = super(ObjectPermissionFormNewUsers, self).clean()
        
        if 'grantee' in data:
            grantee = data['grantee']
            perms = data['permissions']
            
            if 'obj' in data:
                # if grantee does not have permissions, then this is a new user:
                #    - permissions must be selected
                old_perms = grantee.get_perms(data['obj'], groups=False)
                if old_perms:
                    # not new, has perms already
                    data['new'] = False
                    
                elif not perms:
                    # new, doesn't have perms specified
                    msg = """You must grant at least 1 permission for new users and groups"""
                    self._errors["permissions"] = self.error_class([msg])
                    
                else:
                    # new, perms specified
                    data['new'] = True
            
            else:
                # no obj specified, must be adding a new object.  Still need to
                # verify perms
                if not perms:
                    msg = """You must grant at least 1 permission for new users and groups"""
                    self._errors["permissions"] = self.error_class([msg])
        
        return data


def view_users(request, object_, url, \
               template='object_permissions/permissions/users.html', rest=False):
    """
    Generic view for rendering a list of Users who have permissions on an
    object.
    
    This view does not perform any validation of user permissions, that should
    be done in another view which calls this view for display
    
    @param request: HttpRequest
    @param object_: object to list Users and Groups for
    @param url: base url for editing permissions
    @param template: template for rendering User/Group list.
    """
    users = get_users(object_, groups=False)
    groups = get_groups(object_)

    if not rest:
        return render_to_response(template, \
            {'object': object_,
             'users':users,
             'groups':groups,
             'url':url},
        context_instance=RequestContext(request),
    )
    else:
        return {'object': object_,
             'users':users,
             'groups':groups,
             'url':url}


def view_permissions(request, obj, url, user_id=None, group_id=None,
                user_template='object_permissions/permissions/user_row.html',
                group_template='object_permissions/permissions/group_row.html'
                ):
    """
    Update a User or Group permissions on an object.  This is a generic view
    intended to be used for editing permissions on any object.  It must be
    configured with a model and url.  It may also be customized by adding custom
    templates or changing the pk field.
    
    @param obj: object permissions are being set on
    @param url: name of url being edited
    @param user_id: ID of User being edited
    @param group_id: ID of Group being edited
    @param user_template: template used to render user rows
    @param group_template: template used to render group rows
    """
    if request.method == 'POST':
        form = ObjectPermissionFormNewUsers(obj.__class__, request.POST)
        if form.is_valid():
            data = form.cleaned_data
            form_user = form.cleaned_data['user']
            group = form.cleaned_data['group']
            edited_user = form_user if form_user else group
            
            if form.update_perms():
                # send correct signal based on new or edited user
                if data['new']:
                    view_add_user.send(sender=obj.__class__,
                                       editor=request.user,
                                       user=edited_user, obj=obj)
                else:
                    view_edit_user.send(sender=obj.__class__,
                                        editor=request.user,
                                        user=edited_user, obj=obj)
                
                # return html to replace existing user row
                if form_user:
                    return render_to_response(user_template,
                                {'object':obj, 'user_detail':form_user, 'url':url},
                                context_instance=RequestContext(request))
                else:
                    return render_to_response(group_template,
                                {'object':obj, 'group':group, 'url':url},
                                context_instance=RequestContext(request))
                
            else:
                # no permissions, send ajax response to remove user
                view_remove_user.send(sender=obj.__class__,
                                      editor=request.user, user=edited_user,
                                      obj=obj)
                id = ('"user_%d"' if form_user else '"group_%d"')%edited_user.pk
                return HttpResponse(id, mimetype='application/json')

        # error in form return ajax response
        content = json.dumps(form.errors)
        return HttpResponse(content, mimetype='application/json')

    if user_id:
        form_user = get_object_or_404(User, id=user_id)
        data = {'permissions':get_user_perms(form_user, obj, False),
                'user':user_id, 'obj':obj}
    elif group_id:
        group = get_object_or_404(Group, id=group_id)
        data = {'permissions':get_group_perms(group, obj),
                'group':group_id, 'obj':obj}
    else:
        data = {}
        
    form = ObjectPermissionFormNewUsers(obj.__class__, data)
    
    return render_to_response('object_permissions/permissions/form.html',
                {'form':form, 'obj':obj, 'user_id':user_id,
                'group_id':group_id, 'url':url},
               context_instance=RequestContext(request))


@login_required
def view_obj_permissions(request, class_name, obj_id=None,
    user_id=None, group_id=None,
    row_template='object_permissions/permissions/object_row.html'):
    """
    Generic view for editing permissions on an object when the user is already.
    Known.  This is an admin only view since it is impossible to know the
    permission scheme for the apps that are registering properties.
    """
    
    if not request.user.is_superuser:
        return HttpResponseForbidden('You are not authorized to view this page')
    
    try:
        cls = get_class(class_name)
    except KeyError:
        return HttpResponseNotFound('Class type does not exist')
    
    if request.method == 'POST':
        form = ObjectPermissionFormNewUsers(cls, request.POST)
        if form.is_valid():
            data = form.cleaned_data
            form_user = form.cleaned_data['user']
            group = form.cleaned_data['group']
            edited_user = form_user if form_user else group
            
            
            if form.update_perms():
                # send correct signal based on new or edited user
                if data['new']:
                    view_add_user.send(sender=cls,
                                       editor=request.user,
                                       user=edited_user, obj=data['obj'])
                else:
                    view_edit_user.send(sender=cls,
                                        editor=request.user,
                                        user=edited_user, obj=data['obj'])
                
                # return html to replace existing user row
                return render_to_response(row_template,
                    {'class_name':class_name, 'obj':data['obj'], 'persona':edited_user})
            else:
                # no permissions, send ajax response to remove object
                view_remove_user.send(sender=cls,
                                      editor=request.user, user=edited_user,
                                      obj=data['obj'])
                id = '"%s_%s"' % (class_name, obj_id)
                return HttpResponse(id, mimetype='application/json')
        
        # error in form return ajax response
        content = json.dumps(form.errors)
        return HttpResponse(content, mimetype='application/json')
    
    # GET - create form for editing and return as html
    if obj_id:
        obj = get_object_or_404(cls, pk=obj_id)
        data = {'obj':obj}
        if user_id:
            form_user = get_object_or_404(User, id=user_id)
            data['user'] = user_id
            data['permissions'] = get_user_perms(form_user, obj, False)
            url = reverse('user-edit-permissions',
                          args=(user_id, class_name, obj_id))
        elif group_id:
            group = get_object_or_404(Group, id=group_id)
            data['group'] = group_id
            data['permissions'] = get_group_perms(group, obj)
            url = reverse('group-edit-permissions',
                          args=(group_id, class_name, obj_id))
    else:
        obj = None
        if user_id:
            get_object_or_404(User, id=user_id)
            data={'user':user_id}
            url = reverse('user-add-permissions',
                          args=(user_id, class_name))
        elif group_id:
            get_object_or_404(Group, id=group_id)
            data={'group':group_id}
            url = reverse('group-add-permissions',
                          args=(group_id, class_name))
    
    form = ObjectPermissionFormNewUsers(cls, data)
    return render_to_response('object_permissions/permissions/form.html',
            {'form':form, 'obj':obj, 'user_id':user_id, 'group_id':group_id, 
             'url':url},
            context_instance=RequestContext(request))
    
    

@login_required
def all_permissions(request, id,
                    template="object_permissions/permissions/objects.html"):
    """
    Generic view for displaying permissions on all objects.
    
    @param id: id of user
    @param template: template to render the results with, default is
    permissions/objects.html
    """
    user = request.user
    
    if not user.is_superuser:
        return HttpResponseForbidden('You do not have sufficient privileges')
    
    user_detail = get_object_or_404(User, pk=id)
    perm_dict = user_detail.get_all_objects_any_perms(groups=False)

    # exclude group permissions from this view.  they are treated special
    try:
        del perm_dict[Group]
    except KeyError:
        pass

    # XXX repack perm_dict so that class names are used as keys instead of the
    # classes.  Django templates will automatically execute anything callable
    # if you try to use it, even classes!  #5619
    repacked = {}
    for cls, objs in perm_dict.items():
        repacked[cls.__name__] = objs
    
    return render_to_response(template,
            {'persona':user_detail, 'perm_dict':repacked},
        context_instance=RequestContext(request),
    )


from django.contrib.auth.models import Group

from object_permissions import register

# register internal perms
GROUP_PARAMS = {
    'perms':{
        'admin':{
            'label':'Admin',
            'description':'Can add or remove users, and edit their permissions for the group'
        }
    }
}

register(GROUP_PARAMS, Group, 'object_permissions')


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from object_permissions.migrations import db_table_exists

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        # delete old boolean perm columns
        db.delete_column('object_permissions_group_perms', 'admin_tmp')
        if db_table_exists('object_permissions_testmodel_perms'):
            db.delete_column('object_permissions_testmodel_perms', 'Perm3_tmp')
            db.delete_column('object_permissions_testmodel_perms', 'Perm2_tmp')
            db.delete_column('object_permissions_testmodel_perms', 'Perm1_tmp')
            db.delete_column('object_permissions_testmodel_perms', 'Perm4_tmp')
        if db_table_exists('object_permissions_testmodelchildchild_perms'):
            db.delete_column('object_permissions_testmodelchildchild_perms', 'Perm3_tmp')
            db.delete_column('object_permissions_testmodelchildchild_perms', 'Perm2_tmp')
            db.delete_column('object_permissions_testmodelchildchild_perms', 'Perm1_tmp')
            db.delete_column('object_permissions_testmodelchildchild_perms', 'Perm4_tmp')
        if db_table_exists('object_permissions_testmodelchild_perms'):
            db.delete_column('object_permissions_testmodelchild_perms', 'Perm3_tmp')
            db.delete_column('object_permissions_testmodelchild_perms', 'Perm2_tmp')
            db.delete_column('object_permissions_testmodelchild_perms', 'Perm1_tmp')
            db.delete_column('object_permissions_testmodelchild_perms', 'Perm4_tmp')
    
    def backwards(self, orm):
        db.add_column('object_permissions_group_perms', 'admin_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
        if db_table_exists('object_permissions_testmodel_perms'):
            db.add_column('object_permissions_testmodel_perms', 'Perm3_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodel_perms', 'Perm2_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodel_perms', 'Perm1_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodel_perms', 'Perm4_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
        if db_table_exists('object_permissions_testmodelchildchild_perms'):
            db.add_column('object_permissions_testmodelchildchild_perms', 'Perm3_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodelchildchild_perms', 'Perm2_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodelchildchild_perms', 'Perm1_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodelchildchild_perms', 'Perm4_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
        if db_table_exists('object_permissions_testmodelchild_perms'):
            db.add_column('object_permissions_testmodelchild_perms', 'Perm3_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodelchild_perms', 'Perm2_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodelchild_perms', 'Perm1_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))
            db.add_column('object_permissions_testmodelchild_perms', 'Perm4_tmp', self.gf('django.db.models.fields.IntegerField')(default=True))

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'object_permissions.group_perms': {
            'Meta': {'object_name': 'Group_Perms'},
            'admin': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'Group_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['auth.Group']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'Group_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'object_permissions.testmodel': {
            'Meta': {'object_name': 'TestModel'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '32'})
        },
        'object_permissions.testmodel_perms': {
            'Meta': {'object_name': 'TestModel_Perms'},
            'Perm1': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm2': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm3': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm4': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModel_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['object_permissions.TestModel']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModel_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'object_permissions.testmodelchild': {
            'Meta': {'object_name': 'TestModelChild'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['object_permissions.TestModel']", 'null': 'True'})
        },
        'object_permissions.testmodelchild_perms': {
            'Meta': {'object_name': 'TestModelChild_Perms'},
            'Perm1': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm2': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm3': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm4': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChild_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['object_permissions.TestModelChild']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChild_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'object_permissions.testmodelchildchild': {
            'Meta': {'object_name': 'TestModelChildChild'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['object_permissions.TestModelChild']", 'null': 'True'})
        },
        'object_permissions.testmodelchildchild_perms': {
            'Meta': {'object_name': 'TestModelChildChild_Perms'},
            'Perm1': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm2': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm3': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'Perm4': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChildChild_gperms'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'obj': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operms'", 'to': "orm['object_permissions.TestModelChildChild']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'TestModelChildChild_uperms'", 'null': 'True', 'to': "orm['auth.User']"})
        }
    }
    
    complete_apps = ['object_permissions']
